using System;
using System.Data;
using System.Configuration;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Security.Cryptography;
using System.Text;

/// <summary>
/// Summary description for PaymentFormHelper
/// </summary>
namespace MyCompanyName
{
    public class PaymentFormHelper
    {
        public static string GetSiteSecureBaseURL(HttpRequest hrHTTPRequest)
        {
            string szReturnString = null;
            int nIndex;

            szReturnString = hrHTTPRequest.Url.AbsoluteUri;

            nIndex = szReturnString.LastIndexOf('/');

            szReturnString = szReturnString.Substring(0, nIndex + 1);

            return (szReturnString);
        }
        public static byte[] StringToByteArray(string str)
        {
            System.Text.UTF8Encoding encoding = new System.Text.UTF8Encoding();
            return encoding.GetBytes(str);
        }
        public static string ByteArrayToHexString(byte[] aByte)
        {
            StringBuilder sbStringBuilder;
            int nCount = 0;

            sbStringBuilder = new StringBuilder();
            for (nCount = 0; nCount < aByte.Length; nCount++)
            {
                sbStringBuilder.Append(aByte[nCount].ToString("x2"));
            }

            return (sbStringBuilder.ToString());
        }
        public static string ForwardPaddedNumberString(int nNumber, int nPaddingAmount, char cPaddingChar)
        {
            string szReturnString;
            StringBuilder sbString;
            int nCount = 0;

            szReturnString = nNumber.ToString();

            if (szReturnString.Length < nPaddingAmount &&
                nPaddingAmount > 0)
            {
                sbString = new StringBuilder();

                for (nCount = 0; nCount < nPaddingAmount - szReturnString.Length; nCount++)
                {
                    sbString.Append(cPaddingChar);
                }
                sbString.Append(szReturnString);
                szReturnString = sbString.ToString();
            }
            return (szReturnString);
        }
        public static string StripAllWhitespace(string szString)
        {
            StringBuilder sbReturnString;
            int nCount = 0;

            if (szString == null)
            {
                return (null);
            }

            sbReturnString = new StringBuilder();

            for (nCount = 0; nCount < szString.Length; nCount++)
            {
                if (szString[nCount] != ' ' &&
                    szString[nCount] != '\t' &&
                    szString[nCount] != '\n' &&
                    szString[nCount] != '\r')
                {
                    sbReturnString.Append(szString[nCount]);
                }
            }
            return (sbReturnString.ToString());
        }
        public static string CalculateHashDigest(string szInputString)
        {
            byte[] abBytes;
            byte[] abHashDigest;
            MD5 md5;

            abBytes = StringToByteArray(szInputString);

            md5 = new MD5CryptoServiceProvider();
            abHashDigest = md5.ComputeHash(abBytes);

            return (ByteArrayToHexString(abHashDigest));
        }

        public static string GenerateStringToHash(string szAmount, 
                                                  string szCurrencyShort, 
                                                  string szOrderID,
                                                  string szOrderDescription,
                                                  string szSecretKey)
        {
            StringBuilder sbReturnString;

            sbReturnString = new StringBuilder();

            sbReturnString.AppendFormat("Amount={0}&CurrencyShort={1}&OrderID={2}&OrderDescription={3}&SecretKey={4}", szAmount, szCurrencyShort, szOrderID, szOrderDescription, szSecretKey);

            return (sbReturnString.ToString());
        }
        public static string GenerateStringToHash(string szCRES,
                                                  string szCrossReference,
                                                  string szSecretKey)
        {
            StringBuilder sbReturnString;

            sbReturnString = new StringBuilder();

            sbReturnString.AppendFormat("CRES={0}&CrossReference={1}&SecretKey={2}", szCRES, szCrossReference, szSecretKey);

            return (sbReturnString.ToString());
        }
        public static string GenerateStringToHash(string szThreeDSMethodData,
                                                  string szSecretKey)
        {
            StringBuilder sbReturnString;

            sbReturnString = new StringBuilder();

            sbReturnString.AppendFormat("szThreeDSMethodData={0}&SecretKey={1}", szThreeDSMethodData, szSecretKey);

            return (sbReturnString.ToString());
        }

        // This is a "hook" function that is run when the results of the transaction are
        // known. This will be run is 2 places:
        // 1) After the initial CardDetailsTransaction
        // 2) After the ThreeDSecureAuthentication transaction (if 3DS was required)
        //
        // IMPORTANT: in case 1) the unique key is set to be the OrderID, and this will allow
        // your system to lookup the transaction (in your database) if need be, so you need to ensure
        // that your orders are reference-able by their order ids
        //
        // Also, in case 2) the order id field is not avaiable, so the cross reference of the
        // previous "3DS authentication required" transaction is used as the unique key for
        // the transactioon. In order to get this working, you must ensure that when a transaction
        // is set to "3DS auth required" (i.e. the status code is 3), that you update that transaction
        // so that it can be referenced by the cross refernce of the "3DS required" response
        public static void ReportTransactionResults(string szTransactionUniqueKey, int nStatusCode, string szMessage, string szCrossReference)
        {
            if (nStatusCode == 3)
            {
                // you must update the transaction with the szCrossReference field, so that when
                // this function is called after the 3DS authentication is done, the transaction
                // is referenceable by that cross reference - as the subsequent post-3DS call to this
                // function of the transaction will use that cross reference as the transaction's
                // TransactionUniqueKey
            }
        }
    }
}
