using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using ThePaymentGateway.PaymentSystem;
using ThePaymentGateway.Common;

namespace MyCompanyName
{
    public partial class PaymentForm : System.Web.UI.Page
    {
        private enum FORM_MODE
        {
            PAYMENT_FORM,
            THREE_D_SECURE_FINGERPRINT,
            THREE_D_SECURE_CHALLENGE,
            RESULTS
        }
        private enum INCOMING_VARIABLE_SOURCE
        {
            SHOPPING_CART_CHECKOUT,
            PAYMENT_FORM_POSTBACK,
            THREE_D_SECURE_FINGERPRINT,
            THREE_D_SECURE_CHALLENGE
        }

        private FORM_MODE m_fmFormMode = FORM_MODE.PAYMENT_FORM;
        private bool m_boIncomingVariableErrors = false;
        protected internal string m_szMethodDataURL;
        protected internal string m_szThreeDSMethodData;
        protected internal string m_szACSURL;
        protected internal string m_szThreeDSSessionData;
        protected internal string m_szCREQ;

        protected void Page_Load(object sender, EventArgs e)
        {
            string szStringToHash;

            if (IsPostBack)
            {
                if (!CheckIntegrityOfIncomingVariables(INCOMING_VARIABLE_SOURCE.PAYMENT_FORM_POSTBACK))
                {
                    m_boIncomingVariableErrors = true;
                    m_fmFormMode = FORM_MODE.RESULTS;
                    pnMessagePanel.CssClass = "ErrorMessage";
                    pnMessagePanel.Visible = true;
                    lbMessageLabel.Text = "Variable tampering detected";
                    pnTransactionResultsPanel.Visible = false;
                }
            }
            else
            {
                if (Request.Form.Count == 0)
                {
                    m_fmFormMode = FORM_MODE.RESULTS;
                    m_boIncomingVariableErrors = true;
                    pnMessagePanel.CssClass = "ErrorMessage";
                    lbMessageLabel.Text = "No input variables passed";
                    pnMessagePanel.Visible = true;
                    pnTransactionResultsPanel.Visible = false;
                }
                else
                {
                    if (Request.Form["hfThreeDSMethodData"] != null)
                    {
                        if (!CheckIntegrityOfIncomingVariables(INCOMING_VARIABLE_SOURCE.THREE_D_SECURE_FINGERPRINT))
                        {
                            m_boIncomingVariableErrors = true;
                            m_fmFormMode = FORM_MODE.RESULTS;
                            pnMessagePanel.CssClass = "ErrorMessage";
                            lbMessageLabel.Text = "Variable tampering detected";
                            pnMessagePanel.Visible = true;
                            pnTransactionResultsPanel.Visible = false;
                        }
                        else
                        {
                            PerformThreeDSecureEnvironment(Request.Form["hfThreeDSMethodData"], Request.Cookies["cr"].Value);
                        }
                    }
                    // need to work out whether to try the 3d secure authentication
                    else if (Request.Form["hfCRES"] != null &&
                             Request.Form["hfCrossReference"] != null)
                    {
                        if (!CheckIntegrityOfIncomingVariables(INCOMING_VARIABLE_SOURCE.THREE_D_SECURE_CHALLENGE))
                        {
                            m_boIncomingVariableErrors = true;
                            m_fmFormMode = FORM_MODE.RESULTS;
                            pnMessagePanel.CssClass = "ErrorMessage";
                            lbMessageLabel.Text = "Variable tampering detected";
                            pnMessagePanel.Visible = true;
                            pnTransactionResultsPanel.Visible = false;
                        }
                        else
                        {
                            PerformThreeDSecureAuthentication(Request.Form["hfCRES"], Request.Form["hfCrossReference"]);
                        }
                    }
                    else
                    {
                        if (!CheckIntegrityOfIncomingVariables(INCOMING_VARIABLE_SOURCE.SHOPPING_CART_CHECKOUT))
                        {
                            m_boIncomingVariableErrors = true;
                            m_fmFormMode = FORM_MODE.RESULTS;
                            pnMessagePanel.CssClass = "ErrorMessage";
                            lbMessageLabel.Text = "Variable tampering detected";
                            pnMessagePanel.Visible = true;
                            pnTransactionResultsPanel.Visible = false;
                        }

                        PopulateFormItems();

                        // build the has digest for this form
                        szStringToHash = PaymentFormHelper.GenerateStringToHash(hfAmount.Value,
                            hfCurrencyShort.Value,
                            hfOrderID.Value,
                            hfOrderDescription.Value,
                            Global.SecretKey);
                        hfHashDigest.Value = PaymentFormHelper.CalculateHashDigest(szStringToHash);
                    }
                }
            }
        }

        private bool CheckIntegrityOfIncomingVariables(INCOMING_VARIABLE_SOURCE ivsIncomingVariableSource)
        {
            bool boReturnValue = false;
            string szStringToHash;

            switch (ivsIncomingVariableSource)
            {
                case INCOMING_VARIABLE_SOURCE.SHOPPING_CART_CHECKOUT:
                    szStringToHash = PaymentFormHelper.GenerateStringToHash(Request.Form["hfShoppingCartAmount"],
                        Request.Form["hfShoppingCartCurrencyShort"],
                        Request.Form["hfShoppingCartOrderID"],
                        Request.Form["hfShoppingCartOrderDescription"],
                        Global.SecretKey);

                    if (Request.Form["hfShoppingCartHashDigest"] == 
                        PaymentFormHelper.CalculateHashDigest(szStringToHash))
                    {
                        boReturnValue = true;
                    }

                    break;
                case INCOMING_VARIABLE_SOURCE.PAYMENT_FORM_POSTBACK:
                    szStringToHash = PaymentFormHelper.GenerateStringToHash(hfAmount.Value,
                        hfCurrencyShort.Value,
                        hfOrderID.Value,
                        hfOrderDescription.Value,
                        Global.SecretKey);

                    if (hfHashDigest.Value ==
                        PaymentFormHelper.CalculateHashDigest(szStringToHash))
                    {
                        boReturnValue = true;
                    }

                    break;
                case INCOMING_VARIABLE_SOURCE.THREE_D_SECURE_FINGERPRINT:
                    szStringToHash = PaymentFormHelper.GenerateStringToHash(Request.Form["hfThreeDSMethodData"],
                        Global.SecretKey);

                    if (Request.Form["hfShoppingCartHashDigest"] ==
                        PaymentFormHelper.CalculateHashDigest(szStringToHash))
                    {
                        boReturnValue = true;
                    }

                    break;
                case INCOMING_VARIABLE_SOURCE.THREE_D_SECURE_CHALLENGE:
                    szStringToHash = PaymentFormHelper.GenerateStringToHash(Request.Form["hfCRES"],
                        Request.Form["hfCrossReference"],
                        Global.SecretKey);

                    if (Request.Form["hfShoppingCartHashDigest"] ==
                        PaymentFormHelper.CalculateHashDigest(szStringToHash))
                    {
                        boReturnValue = true;
                    }

                    break;
                default:
                    throw new Exception("Invalid incoming variable source: " + ivsIncomingVariableSource.ToString());
            }

            return (boReturnValue);
        }

        protected override void OnPreRender(EventArgs e)
        {
            base.OnPreRender(e);

            // need to work out what mode to put the form into
            switch (m_fmFormMode)
            {
                case FORM_MODE.PAYMENT_FORM:
                    PaymentFormMultiView.SetActiveView(PaymentFormView);
                    BodyTag.Attributes["onload"] = "gatherBrowserData();";
                    break;
                case FORM_MODE.THREE_D_SECURE_FINGERPRINT:
                    PaymentFormMultiView.SetActiveView(ThreeDSecureView);

                    threeDSecureFrame.Attributes["src"] = PaymentFormHelper.GetSiteSecureBaseURL(Request) + "Loading.htm";
                    BodyTag.Attributes["onload"] = "document.threeDSecureEnvironmentForm.submit();";
                    break;
                case FORM_MODE.THREE_D_SECURE_CHALLENGE:
                    PaymentFormMultiView.SetActiveView(ThreeDSecureView);

                    threeDSecureFrame.Attributes["src"] = PaymentFormHelper.GetSiteSecureBaseURL(Request) + "Loading.htm";
                    BodyTag.Attributes["onload"] = "document.threeDSecureChallengeForm.submit();";
                    break;
                case FORM_MODE.RESULTS:
                    PaymentFormMultiView.SetActiveView(ResultsView);
                    break;
                default:
                    throw new Exception("Invalid form mode: " + m_fmFormMode.ToString());
            }

            hfFormMode.Value = m_fmFormMode.ToString();
        }

        private void PerformThreeDSecureAuthentication(string szCRES, string szCrossReference)
        {
            string szMessage = null;
            bool boTransactionSuccessful = false;
            StringBuilder sbString;
            int nCount = 0;
            ThreeDSecureAuthentication tdsaThreeDSecureAuthentication;
            string szPreviousTransactionMessage = null;
            bool boDuplicateTransaction = false;
            string szResponseCrossReference;

            if (m_boIncomingVariableErrors == true)
            {
                return;
            }

            tdsaThreeDSecureAuthentication = new ThreeDSecureAuthentication();
            // you need to put the correct gateway entry point urls in here
            // contact support to get the correct urls

            // The actual values to use for the entry points can be established in a number of ways
            // 1) By periodically issuing a call to GetGatewayEntryPoints
            // 2) By storing the values for the entry points returned with each transaction
            // 3) Speculatively firing transactions at https://gw1.xxx followed by gw2, gw3, gw4....
            // The lower the metric (2nd parameter) means that entry point will be attempted first,
            // EXCEPT if it is -1 - in this case that entry point will be skipped
            // NOTE: You do NOT have to add the entry points in any particular order - the list is sorted
            // by metric value before the transaction sumbitting process begins
            // The 3rd parameter is a retry attempt, so it is possible to try that entry point that number of times
            // before failing over onto the next entry point in the list
            tdsaThreeDSecureAuthentication.RequestGatewayEntryPoints.Add("https://gw1." + Global.PaymentProcessorFullDomain, 100, 1);
            tdsaThreeDSecureAuthentication.RequestGatewayEntryPoints.Add("https://gw2." + Global.PaymentProcessorFullDomain, 200, 1);
            tdsaThreeDSecureAuthentication.RequestGatewayEntryPoints.Add("https://gw3." + Global.PaymentProcessorFullDomain, 300, 1);

            tdsaThreeDSecureAuthentication.MerchantAuthentication.MerchantID = Global.MerchantID;
            tdsaThreeDSecureAuthentication.MerchantAuthentication.Password = Global.Password;
            
            tdsaThreeDSecureAuthentication.ThreeDSecureInputData.CrossReference = szCrossReference;
            tdsaThreeDSecureAuthentication.ThreeDSecureInputData.CRES = szCRES;

            // send the SOAP request
            if (!tdsaThreeDSecureAuthentication.ProcessTransaction())
            {
                szMessage = "Couldn't communicate with payment gateway";
                boTransactionSuccessful = true;

                PaymentFormHelper.ReportTransactionResults(szCrossReference, 30, szMessage, null);
            }
            else
            {
                switch (tdsaThreeDSecureAuthentication.ThreeDSecureAuthenticationResult.StatusCode)
                {
                    case 0:
                        // status code of 0 - means transaction successful 
                        boTransactionSuccessful = true;
                        m_fmFormMode = FORM_MODE.RESULTS;
                        szMessage = tdsaThreeDSecureAuthentication.ThreeDSecureAuthenticationResult.Message;
                        PaymentFormHelper.ReportTransactionResults(szCrossReference, tdsaThreeDSecureAuthentication.ThreeDSecureAuthenticationResult.StatusCode, tdsaThreeDSecureAuthentication.ThreeDSecureAuthenticationResult.Message, tdsaThreeDSecureAuthentication.TransactionOutputData.CrossReference);
                        break;
                    case 5:
                        // status code of 5 - means transaction declined 
                        boTransactionSuccessful = false;
                        m_fmFormMode = FORM_MODE.RESULTS;
                        szMessage = tdsaThreeDSecureAuthentication.ThreeDSecureAuthenticationResult.Message;
                        PaymentFormHelper.ReportTransactionResults(szCrossReference, tdsaThreeDSecureAuthentication.ThreeDSecureAuthenticationResult.StatusCode, tdsaThreeDSecureAuthentication.ThreeDSecureAuthenticationResult.Message, tdsaThreeDSecureAuthentication.TransactionOutputData.CrossReference); 
                        break;
                    case 20:
                        // status code of 20 - means duplicate transaction 
                        m_fmFormMode = FORM_MODE.RESULTS;
                        szMessage = tdsaThreeDSecureAuthentication.ThreeDSecureAuthenticationResult.Message;
                        if (tdsaThreeDSecureAuthentication.ThreeDSecureAuthenticationResult.PreviousTransactionResult.StatusCode.Value == 0)
                        {
                            boTransactionSuccessful = true;
                        }
                        else
                        {
                            boTransactionSuccessful = false;
                        }
                        szPreviousTransactionMessage = tdsaThreeDSecureAuthentication.ThreeDSecureAuthenticationResult.PreviousTransactionResult.Message;
                        boDuplicateTransaction = true;

                        PaymentFormHelper.ReportTransactionResults(szCrossReference, tdsaThreeDSecureAuthentication.ThreeDSecureAuthenticationResult.PreviousTransactionResult.StatusCode.Value, tdsaThreeDSecureAuthentication.ThreeDSecureAuthenticationResult.PreviousTransactionResult.Message, tdsaThreeDSecureAuthentication.TransactionOutputData.CrossReference);
                        break;
                    case 30:
                        // status code of 30 - means an error occurred 
                        boTransactionSuccessful = false;
                        m_fmFormMode = FORM_MODE.RESULTS;

                        sbString = new StringBuilder();

                        // get any additional messages
                        if (tdsaThreeDSecureAuthentication.ThreeDSecureAuthenticationResult.ErrorMessages.Count > 0)
                        {
                            sbString.Append("<br /><ul>");

                            for (nCount = 0; nCount < tdsaThreeDSecureAuthentication.ThreeDSecureAuthenticationResult.ErrorMessages.Count; nCount++)
                            {
                                sbString.AppendFormat("<li>{0}</li>", tdsaThreeDSecureAuthentication.ThreeDSecureAuthenticationResult.ErrorMessages[nCount]);
                            }
                            sbString.Append("</ul>");
                        }

                        szMessage = tdsaThreeDSecureAuthentication.ThreeDSecureAuthenticationResult.Message + sbString.ToString();

                        if (tdsaThreeDSecureAuthentication.TransactionOutputData == null)
                        {
                            szResponseCrossReference = null;
                        }
                        else
                        {
                            szResponseCrossReference = tdsaThreeDSecureAuthentication.TransactionOutputData.CrossReference;
                        }
                        PaymentFormHelper.ReportTransactionResults(szCrossReference, tdsaThreeDSecureAuthentication.ThreeDSecureAuthenticationResult.StatusCode, tdsaThreeDSecureAuthentication.ThreeDSecureAuthenticationResult.Message, szResponseCrossReference);
                        break;
                    default:
                        // unhandled status code  
                        boTransactionSuccessful = false;
                        m_fmFormMode = FORM_MODE.RESULTS;
                        szMessage = tdsaThreeDSecureAuthentication.ThreeDSecureAuthenticationResult.Message;

                        if (tdsaThreeDSecureAuthentication.TransactionOutputData == null)
                        {
                            szResponseCrossReference = null;
                        }
                        else
                        {
                            szResponseCrossReference = tdsaThreeDSecureAuthentication.TransactionOutputData.CrossReference;
                        }
                        PaymentFormHelper.ReportTransactionResults(szCrossReference, tdsaThreeDSecureAuthentication.ThreeDSecureAuthenticationResult.StatusCode, tdsaThreeDSecureAuthentication.ThreeDSecureAuthenticationResult.Message, szResponseCrossReference);
                        break;
                }
            }

            pnTransactionResultsPanel.Visible = true;
            pnMessagePanel.Visible = false;

            if (!boTransactionSuccessful)
            {
                pnTransactionResultsPanel.CssClass = "ErrorMessage";
            }
            else
            {
                pnTransactionResultsPanel.CssClass = "SuccessMessage";
            }

            lbGatewayResponse.Text = szMessage;

            // sort out the duplicate transaction reporting
            if (boDuplicateTransaction)
            {
                pnDuplicateTransactionPanel.Visible = true;
                lbPreviousTransactionMessage.Text = szPreviousTransactionMessage;
            }
        }

        private void PerformThreeDSecureEnvironment(string szThreeDSMethodData, string szCrossReference)
        {
            string szMessage = null;
            bool boTransactionSuccessful = false;
            StringBuilder sbString;
            int nCount = 0;
            ThreeDSecureEnvironment tdseThreeDSecureEnvironment;
            string szPreviousTransactionMessage = null;
            bool boDuplicateTransaction = false;
            string szResponseCrossReference;

            if (m_boIncomingVariableErrors == true)
            {
                return;
            }

            tdseThreeDSecureEnvironment = new ThreeDSecureEnvironment();
            // you need to put the correct gateway entry point urls in here
            // contact support to get the correct urls

            // The actual values to use for the entry points can be established in a number of ways
            // 1) By periodically issuing a call to GetGatewayEntryPoints
            // 2) By storing the values for the entry points returned with each transaction
            // 3) Speculatively firing transactions at https://gw1.xxx followed by gw2, gw3, gw4....
            // The lower the metric (2nd parameter) means that entry point will be attempted first,
            // EXCEPT if it is -1 - in this case that entry point will be skipped
            // NOTE: You do NOT have to add the entry points in any particular order - the list is sorted
            // by metric value before the transaction sumbitting process begins
            // The 3rd parameter is a retry attempt, so it is possible to try that entry point that number of times
            // before failing over onto the next entry point in the list
            tdseThreeDSecureEnvironment.RequestGatewayEntryPoints.Add("https://gw1." + Global.PaymentProcessorFullDomain, 100, 1);
            tdseThreeDSecureEnvironment.RequestGatewayEntryPoints.Add("https://gw2." + Global.PaymentProcessorFullDomain, 200, 1);
            tdseThreeDSecureEnvironment.RequestGatewayEntryPoints.Add("https://gw3." + Global.PaymentProcessorFullDomain, 300, 1);

            tdseThreeDSecureEnvironment.MerchantAuthentication.MerchantID = Global.MerchantID;
            tdseThreeDSecureEnvironment.MerchantAuthentication.Password = Global.Password;

            tdseThreeDSecureEnvironment.ThreeDSecureEnvironmentData.CrossReference = szCrossReference;
            tdseThreeDSecureEnvironment.ThreeDSecureEnvironmentData.MethodData = szThreeDSMethodData;

            // send the SOAP request
            if (!tdseThreeDSecureEnvironment.ProcessTransaction())
            {
                szMessage = "Couldn't communicate with payment gateway";
                boTransactionSuccessful = true;

                PaymentFormHelper.ReportTransactionResults(szCrossReference, 30, szMessage, null);
            }
            else
            {
                switch (tdseThreeDSecureEnvironment.ThreeDSecureEnvironmentResult.StatusCode)
                {
                    case 0:
                        // status code of 0 - means transaction successful 
                        boTransactionSuccessful = true;
                        m_fmFormMode = FORM_MODE.RESULTS;
                        szMessage = tdseThreeDSecureEnvironment.ThreeDSecureEnvironmentResult.Message;
                        PaymentFormHelper.ReportTransactionResults(szCrossReference, tdseThreeDSecureEnvironment.ThreeDSecureEnvironmentResult.StatusCode, tdseThreeDSecureEnvironment.ThreeDSecureEnvironmentResult.Message, tdseThreeDSecureEnvironment.TransactionOutputData.CrossReference);
                        break;
                    case 3:
                        // status code of 3 - means 3D Secure authentication required 
                        m_fmFormMode = FORM_MODE.THREE_D_SECURE_CHALLENGE;

                        m_szACSURL = tdseThreeDSecureEnvironment.TransactionOutputData.ThreeDSecureOutputData.ACSURL;
                        m_szCREQ = tdseThreeDSecureEnvironment.TransactionOutputData.ThreeDSecureOutputData.CREQ;
                        m_szThreeDSSessionData = tdseThreeDSecureEnvironment.TransactionOutputData.CrossReference;// Request.Cookies["cr"].Value;
                        PaymentFormHelper.ReportTransactionResults(hfOrderID.Value, tdseThreeDSecureEnvironment.ThreeDSecureEnvironmentResult.StatusCode, tdseThreeDSecureEnvironment.ThreeDSecureEnvironmentResult.Message, tdseThreeDSecureEnvironment.TransactionOutputData.CrossReference);
                        break;
                    case 5:
                        // status code of 5 - means transaction declined 
                        boTransactionSuccessful = false;
                        m_fmFormMode = FORM_MODE.RESULTS;
                        szMessage = tdseThreeDSecureEnvironment.ThreeDSecureEnvironmentResult.Message;
                        PaymentFormHelper.ReportTransactionResults(szCrossReference, tdseThreeDSecureEnvironment.ThreeDSecureEnvironmentResult.StatusCode, tdseThreeDSecureEnvironment.ThreeDSecureEnvironmentResult.Message, tdseThreeDSecureEnvironment.TransactionOutputData.CrossReference);
                        break;
                    case 20:
                        // status code of 20 - means duplicate transaction 
                        m_fmFormMode = FORM_MODE.RESULTS;
                        szMessage = tdseThreeDSecureEnvironment.ThreeDSecureEnvironmentResult.Message;
                        if (tdseThreeDSecureEnvironment.ThreeDSecureEnvironmentResult.PreviousTransactionResult.StatusCode.Value == 0)
                        {
                            boTransactionSuccessful = true;
                        }
                        else
                        {
                            boTransactionSuccessful = false;
                        }
                        szPreviousTransactionMessage = tdseThreeDSecureEnvironment.ThreeDSecureEnvironmentResult.PreviousTransactionResult.Message;
                        boDuplicateTransaction = true;

                        PaymentFormHelper.ReportTransactionResults(szCrossReference, tdseThreeDSecureEnvironment.ThreeDSecureEnvironmentResult.PreviousTransactionResult.StatusCode.Value, tdseThreeDSecureEnvironment.ThreeDSecureEnvironmentResult.PreviousTransactionResult.Message, tdseThreeDSecureEnvironment.TransactionOutputData.CrossReference);
                        break;
                    case 30:
                        // status code of 30 - means an error occurred 
                        boTransactionSuccessful = false;
                        m_fmFormMode = FORM_MODE.RESULTS;

                        sbString = new StringBuilder();

                        // get any additional messages
                        if (tdseThreeDSecureEnvironment.ThreeDSecureEnvironmentResult.ErrorMessages.Count > 0)
                        {
                            sbString.Append("<br /><ul>");

                            for (nCount = 0; nCount < tdseThreeDSecureEnvironment.ThreeDSecureEnvironmentResult.ErrorMessages.Count; nCount++)
                            {
                                sbString.AppendFormat("<li>{0}</li>", tdseThreeDSecureEnvironment.ThreeDSecureEnvironmentResult.ErrorMessages[nCount]);
                            }
                            sbString.Append("</ul>");
                        }

                        szMessage = tdseThreeDSecureEnvironment.ThreeDSecureEnvironmentResult.Message + sbString.ToString();

                        if (tdseThreeDSecureEnvironment.TransactionOutputData == null)
                        {
                            szResponseCrossReference = null;
                        }
                        else
                        {
                            szResponseCrossReference = tdseThreeDSecureEnvironment.TransactionOutputData.CrossReference;
                        }
                        PaymentFormHelper.ReportTransactionResults(szCrossReference, tdseThreeDSecureEnvironment.ThreeDSecureEnvironmentResult.StatusCode, tdseThreeDSecureEnvironment.ThreeDSecureEnvironmentResult.Message, szResponseCrossReference);
                        break;
                    default:
                        // unhandled status code  
                        boTransactionSuccessful = false;
                        m_fmFormMode = FORM_MODE.RESULTS;
                        szMessage = tdseThreeDSecureEnvironment.ThreeDSecureEnvironmentResult.Message;

                        if (tdseThreeDSecureEnvironment.TransactionOutputData == null)
                        {
                            szResponseCrossReference = null;
                        }
                        else
                        {
                            szResponseCrossReference = tdseThreeDSecureEnvironment.TransactionOutputData.CrossReference;
                        }
                        PaymentFormHelper.ReportTransactionResults(szCrossReference, tdseThreeDSecureEnvironment.ThreeDSecureEnvironmentResult.StatusCode, tdseThreeDSecureEnvironment.ThreeDSecureEnvironmentResult.Message, szResponseCrossReference);
                        break;
                }
            }

            pnTransactionResultsPanel.Visible = true;
            pnMessagePanel.Visible = false;

            if (!boTransactionSuccessful)
            {
                pnTransactionResultsPanel.CssClass = "ErrorMessage";
            }
            else
            {
                pnTransactionResultsPanel.CssClass = "SuccessMessage";
            }

            lbGatewayResponse.Text = szMessage;

            // sort out the duplicate transaction reporting
            if (boDuplicateTransaction)
            {
                pnDuplicateTransactionPanel.Visible = true;
                lbPreviousTransactionMessage.Text = szPreviousTransactionMessage;
            }
        }

        private void PopulateFormItems()
        {
            string szWorkingValue;
            string szWorkingDisplayValue;
            int nCount = 0;
            int nCurrentYear;
            ListItem liListItem;
            bool boHitFirstZeroPriorityCountry = false;
            ThePaymentGateway.Common.ISOCurrency icISOCurrency;

            if (m_boIncomingVariableErrors == true)
            {
                return;
            }

            // read the values from the shopping cart checkout
            hfAmount.Value = Request.Form["hfShoppingCartAmount"];
            hfCurrencyShort.Value = Request.Form["hfShoppingCartCurrencyShort"];
            hfOrderID.Value = Request.Form["hfShoppingCartOrderID"];
            hfOrderDescription.Value = Request.Form["hfShoppingCartOrderDescription"];

            // populate the label text
            if (!Global.ISOCurrencyData.GetISOCurrency(hfCurrencyShort.Value, out icISOCurrency))
            {
                throw new Exception("Couldn't get ISO currency: " + hfCurrencyShort.Value);
            }
            lbAmount.Text = icISOCurrency.GetAmountCurrencyString(System.Convert.ToInt32(Request.Form["hfShoppingCartAmount"]));
            lbOrderID.Text = Request.Form["hfShoppingCartOrderID"];
            lbOrderDescription.Text = Request.Form["hfShoppingCartOrderDescription"];

            tbAddress1.Text = "";
            tbAddress2.Text = "";
            tbAddress3.Text = "";
            tbAddress4.Text = "";
            tbCity.Text = "";
            tbState.Text = "";
            tbPostCode.Text = "";
            tbCardName.Text = "";
            tbCardNumber.Text = "";
            tbIssueNumber.Text = "";
            tbCV2.Text = "";
            lbMessageLabel.Text = "";
            pnMessagePanel.Visible = false;

            liListItem = new ListItem("", "-1");
            ddlCountryShort.Items.Add(liListItem);

            // populate the country drop down
            for (nCount = 0; nCount < Global.ISOCountryData.Count; nCount++)
            {
                if (nCount > 0 &&
                    !boHitFirstZeroPriorityCountry &&
                    Global.ISOCountryData[nCount].ListPriority == 0)
                {
                    boHitFirstZeroPriorityCountry = true;
                    liListItem = new ListItem("---------------------", "-1");
                    ddlCountryShort.Items.Add(liListItem);
                }
                liListItem = new ListItem(Global.ISOCountryData[nCount].CountryName, Global.ISOCountryData[nCount].CountryShort);

                ddlCountryShort.Items.Add(liListItem);
            }

            nCurrentYear = DateTime.Now.Year;

            ddExpiryDateYear.Items.Add("");
            ddExpiryDateMonth.Items.Add("");

            ddStartDateYear.Items.Add("");
            ddStartDateMonth.Items.Add("");

            for (nCount = 0; nCount < 10; nCount++)
            {
                // expiry date - start with this year & add 10
                szWorkingDisplayValue = System.Convert.ToString(nCurrentYear + nCount);
                szWorkingValue = szWorkingDisplayValue.Substring(2);
                liListItem = new ListItem(szWorkingDisplayValue, szWorkingValue);
                ddExpiryDateYear.Items.Add(liListItem);

                // start date - start with this year & minus 10
                szWorkingDisplayValue = System.Convert.ToString(nCurrentYear - nCount);
                szWorkingValue = szWorkingDisplayValue.Substring(2);
                liListItem = new ListItem(szWorkingDisplayValue, szWorkingValue);
                ddStartDateYear.Items.Add(liListItem);
            }

            for (nCount = 1; nCount <= 12; nCount++)
            {
                szWorkingValue = System.Convert.ToString(nCount);
                szWorkingDisplayValue = MyCompanyName.PaymentFormHelper.ForwardPaddedNumberString(nCount, 2, '0');
                liListItem = new ListItem(szWorkingDisplayValue, szWorkingValue);
                ddExpiryDateMonth.Items.Add(liListItem);

                liListItem = new ListItem(szWorkingDisplayValue, szWorkingValue);
                ddStartDateMonth.Items.Add(liListItem);
            }
        }

        protected void SubmitButton_Click(object sender, EventArgs e)
        {
            string szMessage = null;
            bool boTransactionSuccessful = false;
            StringBuilder sbString;
            int nCount = 0;
            CardDetailsTransaction cdtCardDetailsTransaction;
            string szPreviousTransactionMessage = null;
            bool boDuplicateTransaction = false;
            string szCrossReference;
            ISOCountry icISOCountry;
            ISOCurrency icISOCurrency;

            if (m_boIncomingVariableErrors == true)
            {
                return;
            }

            cdtCardDetailsTransaction = new CardDetailsTransaction();
            // you need to put the correct gateway entry point urls in here
            // contact support to get the correct urls

            // The actual values to use for the entry points can be established in a number of ways
            // 1) By periodically issuing a call to GetGatewayEntryPoints
            // 2) By storing the values for the entry points returned with each transaction
            // 3) Speculatively firing transactions at https://gw1.xxx followed by gw2, gw3, gw4....
            // The lower the metric (2nd parameter) means that entry point will be attempted first,
            // EXCEPT if it is -1 - in this case that entry point will be skipped
            // NOTE: You do NOT have to add the entry points in any particular order - the list is sorted
            // by metric value before the transaction sumbitting process begins
            // The 3rd parameter is a retry attempt, so it is possible to try that entry point that number of times
            // before failing over onto the next entry point in the list
            cdtCardDetailsTransaction.RequestGatewayEntryPoints.Add("https://gw1." + Global.PaymentProcessorFullDomain, 100, 1);
            cdtCardDetailsTransaction.RequestGatewayEntryPoints.Add("https://gw2." + Global.PaymentProcessorFullDomain, 200, 1);
            cdtCardDetailsTransaction.RequestGatewayEntryPoints.Add("https://gw3." + Global.PaymentProcessorFullDomain, 300, 1);

            cdtCardDetailsTransaction.MerchantAuthentication.MerchantID = Global.MerchantID;
            cdtCardDetailsTransaction.MerchantAuthentication.Password = Global.Password;

            cdtCardDetailsTransaction.TransactionDetails.MessageDetails.TransactionType = "SALE";
            
            cdtCardDetailsTransaction.TransactionDetails.Amount.Value = System.Convert.ToInt32(hfAmount.Value);
            if (!String.IsNullOrEmpty(hfCurrencyShort.Value))
            {
                if (Global.ISOCurrencyData.GetISOCurrency(hfCurrencyShort.Value, out icISOCurrency))
                {
                    cdtCardDetailsTransaction.TransactionDetails.CurrencyCode.Value = icISOCurrency.ISOCode;
                }
            }
            cdtCardDetailsTransaction.TransactionDetails.OrderID = hfOrderID.Value;
            cdtCardDetailsTransaction.TransactionDetails.OrderDescription = hfOrderDescription.Value;

            cdtCardDetailsTransaction.CardDetails.CardName = tbCardName.Text;
            cdtCardDetailsTransaction.CardDetails.CardNumber = tbCardNumber.Text;
            if (!String.IsNullOrEmpty(ddExpiryDateMonth.SelectedValue))
            {
                cdtCardDetailsTransaction.CardDetails.ExpiryDate.Month.Value = System.Convert.ToInt32(ddExpiryDateMonth.SelectedValue);
            }
            if (!String.IsNullOrEmpty(ddExpiryDateYear.SelectedValue))
            {
                cdtCardDetailsTransaction.CardDetails.ExpiryDate.Year.Value = System.Convert.ToInt32(ddExpiryDateYear.SelectedValue);
            }
            if (!String.IsNullOrEmpty(ddStartDateMonth.SelectedValue))
            {
                cdtCardDetailsTransaction.CardDetails.StartDate.Month.Value = System.Convert.ToInt32(ddStartDateMonth.SelectedValue);
            }
            if (!String.IsNullOrEmpty(ddStartDateYear.SelectedValue))
            {
                cdtCardDetailsTransaction.CardDetails.ExpiryDate.Year.Value = System.Convert.ToInt32(ddStartDateYear.SelectedValue);
            }
            cdtCardDetailsTransaction.CardDetails.IssueNumber = tbIssueNumber.Text;
            cdtCardDetailsTransaction.CardDetails.CV2 = tbCV2.Text;

            cdtCardDetailsTransaction.CustomerDetails.EmailAddress = "test@test.com";
            cdtCardDetailsTransaction.CustomerDetails.PhoneNumber = "123456789";
            // the direct/API can only currently handle IPv4 addresses.
            // Uncomment the following line out if the UserHostAddress will carry an IPv4 address
            //cdtCardDetailsTransaction.CustomerDetails.CustomerIPAddress = Request.UserHostAddress;
            cdtCardDetailsTransaction.CustomerDetails.BillingAddress.Address1 = tbAddress1.Text;
            cdtCardDetailsTransaction.CustomerDetails.BillingAddress.Address2 = tbAddress2.Text;
            cdtCardDetailsTransaction.CustomerDetails.BillingAddress.Address3 = tbAddress3.Text;
            cdtCardDetailsTransaction.CustomerDetails.BillingAddress.Address4 = tbAddress4.Text;
            cdtCardDetailsTransaction.CustomerDetails.BillingAddress.City = tbCity.Text;
            cdtCardDetailsTransaction.CustomerDetails.BillingAddress.State = tbState.Text;
            cdtCardDetailsTransaction.CustomerDetails.BillingAddress.PostCode = tbPostCode.Text;
            if (!String.IsNullOrEmpty(ddlCountryShort.SelectedValue) &&
                ddlCountryShort.SelectedValue != "-1")
            {
                if (Global.ISOCountryData.GetISOCountry(ddlCountryShort.SelectedValue, out icISOCountry))
                {
                    cdtCardDetailsTransaction.CustomerDetails.BillingAddress.CountryCode.Value = icISOCountry.ISOCode;
                }
            }

            cdtCardDetailsTransaction.TransactionDetails.TransactionControl.EchoCardType.Value = true;
            cdtCardDetailsTransaction.TransactionDetails.TransactionControl.EchoAVSCheckResult.Value = true;
            cdtCardDetailsTransaction.TransactionDetails.TransactionControl.EchoCV2CheckResult.Value = true;
            cdtCardDetailsTransaction.TransactionDetails.TransactionControl.DuplicateDelay.Value = 60;
            cdtCardDetailsTransaction.TransactionDetails.TransactionControl.ThreeDSecureOverridePolicy.Value = true;
            
            cdtCardDetailsTransaction.TransactionDetails.ThreeDSecureBrowserDetails.DeviceCategory.Value = 0;
            cdtCardDetailsTransaction.TransactionDetails.ThreeDSecureBrowserDetails.AcceptHeaders = "*/*";
            cdtCardDetailsTransaction.TransactionDetails.ThreeDSecureBrowserDetails.UserAgent = Request.UserAgent;

            if (!String.IsNullOrEmpty(hfJavaEnabled.Value))
            {
                cdtCardDetailsTransaction.TransactionDetails.ThreeDSecureBrowserDetails.JavaEnabled.Value = Convert.ToBoolean(hfJavaEnabled.Value);
            }
            if (!String.IsNullOrEmpty(hfJavaScriptEnabled.Value))
            {
                cdtCardDetailsTransaction.TransactionDetails.ThreeDSecureBrowserDetails.JavaScriptEnabled.Value = Convert.ToBoolean(hfJavaScriptEnabled.Value);
            }
            if (!String.IsNullOrEmpty(hfScreenWidth.Value))
            {
                cdtCardDetailsTransaction.TransactionDetails.ThreeDSecureBrowserDetails.ScreenWidth.Value = Convert.ToInt32(hfScreenWidth.Value);
            }
            if (!String.IsNullOrEmpty(hfScreenHeight.Value))
            {
                cdtCardDetailsTransaction.TransactionDetails.ThreeDSecureBrowserDetails.ScreenHeight.Value = Convert.ToInt32(hfScreenHeight.Value);
            }
            if (!String.IsNullOrEmpty(hfScreenColourDepth.Value))
            {
                cdtCardDetailsTransaction.TransactionDetails.ThreeDSecureBrowserDetails.ScreenColourDepth.Value = Convert.ToInt32(hfScreenColourDepth.Value);
            }
            if (!String.IsNullOrEmpty(hfTimezoneOffset.Value))
            {
                cdtCardDetailsTransaction.TransactionDetails.ThreeDSecureBrowserDetails.TimeZone.Value = Convert.ToInt32(hfTimezoneOffset.Value);
            }
            cdtCardDetailsTransaction.TransactionDetails.ThreeDSecureBrowserDetails.Language = hfLanguage.Value;

            cdtCardDetailsTransaction.TransactionDetails.ThreeDSecureNotificationDetails.ChallengeNotificationURL = PaymentFormHelper.GetSiteSecureBaseURL(Request) + "ThreeDSecureLandingPage.aspx";
            cdtCardDetailsTransaction.TransactionDetails.ThreeDSecureNotificationDetails.FingerprintNotificationURL = PaymentFormHelper.GetSiteSecureBaseURL(Request) + "ThreeDSecureLandingPage.aspx";

            // send the SOAP request
            if (!cdtCardDetailsTransaction.ProcessTransaction())
            {
                szMessage = "Couldn't communicate with payment gateway";
                boTransactionSuccessful = false;

                PaymentFormHelper.ReportTransactionResults(hfOrderID.Value, 30, szMessage, null);
            }
            else
            {
                switch (cdtCardDetailsTransaction.CardDetailsTransactionResult.StatusCode)
                {
                    case 0:
				        // status code of 0 - means transaction successful 
                        boTransactionSuccessful = true;
                        m_fmFormMode = FORM_MODE.RESULTS;
                        szMessage = cdtCardDetailsTransaction.CardDetailsTransactionResult.Message;
                        PaymentFormHelper.ReportTransactionResults(hfOrderID.Value, cdtCardDetailsTransaction.CardDetailsTransactionResult.StatusCode, cdtCardDetailsTransaction.CardDetailsTransactionResult.Message, cdtCardDetailsTransaction.TransactionOutputData.CrossReference);
                        break;
                    case 3:
				        // status code of 3 - means 3D Secure authentication required 
                        m_fmFormMode = FORM_MODE.THREE_D_SECURE_FINGERPRINT;

                        m_szMethodDataURL = cdtCardDetailsTransaction.TransactionOutputData.ThreeDSecureOutputData.MethodURL;
                        m_szThreeDSMethodData = cdtCardDetailsTransaction.TransactionOutputData.ThreeDSecureOutputData.MethodData;
                        // the cross reference needs to be stored somewhere where it can be retrieved again later. For this stage, the 3DSv2 spec doesn't have a "pass through" field
                        // that just echoes back what it's given (unlike a later stage with the threeDSSessionData)
                        // for now - just store this in a cookie
                        Response.Cookies.Add(new HttpCookie("cr", cdtCardDetailsTransaction.TransactionOutputData.CrossReference));
                        PaymentFormHelper.ReportTransactionResults(hfOrderID.Value, cdtCardDetailsTransaction.CardDetailsTransactionResult.StatusCode, cdtCardDetailsTransaction.CardDetailsTransactionResult.Message, cdtCardDetailsTransaction.TransactionOutputData.CrossReference);
                        break;
                    case 5:
				        // status code of 5 - means transaction declined 
                        boTransactionSuccessful = false;
                        m_fmFormMode = FORM_MODE.RESULTS;
                        szMessage = cdtCardDetailsTransaction.CardDetailsTransactionResult.Message;
                        PaymentFormHelper.ReportTransactionResults(hfOrderID.Value, cdtCardDetailsTransaction.CardDetailsTransactionResult.StatusCode, cdtCardDetailsTransaction.CardDetailsTransactionResult.Message, cdtCardDetailsTransaction.TransactionOutputData.CrossReference);
                        break;
                    case 20:
				        // status code of 20 - means duplicate transaction 
				        m_fmFormMode = FORM_MODE.RESULTS;
                        szMessage = cdtCardDetailsTransaction.CardDetailsTransactionResult.Message;
                        if (cdtCardDetailsTransaction.CardDetailsTransactionResult.PreviousTransactionResult.StatusCode.Value == 0)
                        {
					        boTransactionSuccessful = true;
				        }
                        else
                        {
					        boTransactionSuccessful = false;
			   	        }
                        szPreviousTransactionMessage = cdtCardDetailsTransaction.CardDetailsTransactionResult.PreviousTransactionResult.Message;
                        boDuplicateTransaction = true;
                        PaymentFormHelper.ReportTransactionResults(hfOrderID.Value, cdtCardDetailsTransaction.CardDetailsTransactionResult.PreviousTransactionResult.StatusCode.Value, szPreviousTransactionMessage, cdtCardDetailsTransaction.TransactionOutputData.CrossReference);
                        break;
                    case 30:
				        // status code of 30 - means an error occurred 
                        boTransactionSuccessful = false;
                        m_fmFormMode = FORM_MODE.PAYMENT_FORM;

                        sbString = new StringBuilder();

                        // get any additional messages
                        if (cdtCardDetailsTransaction.CardDetailsTransactionResult.ErrorMessages.Count > 0)
                        {
                            sbString.Append("<br /><ul>");

                            for (nCount = 0; nCount < cdtCardDetailsTransaction.CardDetailsTransactionResult.ErrorMessages.Count; nCount++)
                            {
                                sbString.AppendFormat("<li>{0}</li>", cdtCardDetailsTransaction.CardDetailsTransactionResult.ErrorMessages[nCount]);
                            }
                            sbString.Append("</ul>");
                        }

                        szMessage = cdtCardDetailsTransaction.CardDetailsTransactionResult.Message + sbString.ToString();

                        if (cdtCardDetailsTransaction.TransactionOutputData == null)
                        {
                            szCrossReference = null;
                        }
                        else
                        {
                            szCrossReference = cdtCardDetailsTransaction.TransactionOutputData.CrossReference;
                        }

                        PaymentFormHelper.ReportTransactionResults(hfOrderID.Value, cdtCardDetailsTransaction.CardDetailsTransactionResult.StatusCode, cdtCardDetailsTransaction.CardDetailsTransactionResult.Message, szCrossReference);
                        break;
                    default:
                        // unhandled status code  
                        boTransactionSuccessful = false;
                        m_fmFormMode = FORM_MODE.PAYMENT_FORM;
                        szMessage = cdtCardDetailsTransaction.CardDetailsTransactionResult.Message;

                        if (cdtCardDetailsTransaction.TransactionOutputData == null)
                        {
                            szCrossReference = null;
                        }
                        else
                        {
                            szCrossReference = cdtCardDetailsTransaction.TransactionOutputData.CrossReference;
                        }

                        PaymentFormHelper.ReportTransactionResults(hfOrderID.Value, cdtCardDetailsTransaction.CardDetailsTransactionResult.StatusCode, cdtCardDetailsTransaction.CardDetailsTransactionResult.Message, szCrossReference);
                        break;
                }
            }

            if (m_fmFormMode == FORM_MODE.PAYMENT_FORM)
            {
                pnMessagePanel.CssClass = "ErrorMessage";
                pnMessagePanel.Visible = true;
                lbMessageLabel.Text = szMessage;
                pnTransactionResultsPanel.Visible = false;
            }
            else
            {
                pnTransactionResultsPanel.Visible = true;
                pnMessagePanel.Visible = false;

                if (!boTransactionSuccessful)
                {
                    pnTransactionResultsPanel.CssClass = "ErrorMessage";
                }
                else
                {
                    pnTransactionResultsPanel.CssClass = "SuccessMessage";
                }
                
                lbGatewayResponse.Text = szMessage;
                
                // sort out the duplicate transaction reporting
                if (boDuplicateTransaction)
                {
                    pnDuplicateTransactionPanel.Visible = true;
                    lbPreviousTransactionMessage.Text = szPreviousTransactionMessage;
                }
            }
        }
    }
}
