Imports System
Imports System.Text
Imports System.Web.Security
Imports System.Web.UI
Imports System.Web.UI.WebControls
Imports System.Web.UI.WebControls.WebParts
Imports System.Web.UI.HtmlControls
Imports ThePaymentGateway.PaymentSystem
Imports ThePaymentGateway.Common

Namespace MyCompanyName
    Public Partial Class PaymentForm
        Inherits Web.UI.Page
        Private Enum FORM_MODE
            PAYMENT_FORM
            THREE_D_SECURE_FINGERPRINT
            THREE_D_SECURE_CHALLENGE
            RESULTS
        End Enum
        Private Enum INCOMING_VARIABLE_SOURCE
            SHOPPING_CART_CHECKOUT
            PAYMENT_FORM_POSTBACK
            THREE_D_SECURE_FINGERPRINT
            THREE_D_SECURE_CHALLENGE
        End Enum

        Private m_fmFormMode As FORM_MODE = FORM_MODE.PAYMENT_FORM
        Private m_boIncomingVariableErrors As Boolean = False
        Protected Friend m_szMethodDataURL As String
        Protected Friend m_szThreeDSMethodData As String
        Protected Friend m_szACSURL As String
        Protected Friend m_szThreeDSSessionData As String
        Protected Friend m_szCREQ As String

        Protected Sub Page_Load(ByVal sender As Object, ByVal e As EventArgs)
            Dim szStringToHash As String

            If IsPostBack Then
                If Not CheckIntegrityOfIncomingVariables(INCOMING_VARIABLE_SOURCE.PAYMENT_FORM_POSTBACK) Then
                    m_boIncomingVariableErrors = True
                    m_fmFormMode = FORM_MODE.RESULTS
                    pnMessagePanel.CssClass = "ErrorMessage"
                    pnMessagePanel.Visible = True
                    lbMessageLabel.Text = "Variable tampering detected"
                    pnTransactionResultsPanel.Visible = False
                End If
            Else
                If Request.Form.Count = 0 Then
                    m_fmFormMode = FORM_MODE.RESULTS
                    m_boIncomingVariableErrors = True
                    pnMessagePanel.CssClass = "ErrorMessage"
                    lbMessageLabel.Text = "No input variables passed"
                    pnMessagePanel.Visible = True
                    pnTransactionResultsPanel.Visible = False
                Else
                    If Request.Form("hfThreeDSMethodData") IsNot Nothing Then
                        If Not CheckIntegrityOfIncomingVariables(INCOMING_VARIABLE_SOURCE.THREE_D_SECURE_FINGERPRINT) Then
                            m_boIncomingVariableErrors = True
                            m_fmFormMode = FORM_MODE.RESULTS
                            pnMessagePanel.CssClass = "ErrorMessage"
                            lbMessageLabel.Text = "Variable tampering detected"
                            pnMessagePanel.Visible = True
                            pnTransactionResultsPanel.Visible = False
                        Else
                            PerformThreeDSecureEnvironment(Request.Form("hfThreeDSMethodData"), Request.Cookies("cr").Value)
                        End If
                        ' need to work out whether to try the 3d secure authentication
                    ElseIf Request.Form("hfCRES") IsNot Nothing AndAlso Request.Form("hfCrossReference") IsNot Nothing Then
                        If Not CheckIntegrityOfIncomingVariables(INCOMING_VARIABLE_SOURCE.THREE_D_SECURE_CHALLENGE) Then
                            m_boIncomingVariableErrors = True
                            m_fmFormMode = FORM_MODE.RESULTS
                            pnMessagePanel.CssClass = "ErrorMessage"
                            lbMessageLabel.Text = "Variable tampering detected"
                            pnMessagePanel.Visible = True
                            pnTransactionResultsPanel.Visible = False
                        Else
                            PerformThreeDSecureAuthentication(Request.Form("hfCRES"), Request.Form("hfCrossReference"))
                        End If
                    Else
                        If Not CheckIntegrityOfIncomingVariables(INCOMING_VARIABLE_SOURCE.SHOPPING_CART_CHECKOUT) Then
                            m_boIncomingVariableErrors = True
                            m_fmFormMode = FORM_MODE.RESULTS
                            pnMessagePanel.CssClass = "ErrorMessage"
                            lbMessageLabel.Text = "Variable tampering detected"
                            pnMessagePanel.Visible = True
                            pnTransactionResultsPanel.Visible = False
                        End If

                        PopulateFormItems()

                        ' build the has digest for this form
                        szStringToHash = PaymentFormHelper.GenerateStringToHash(hfAmount.Value, hfCurrencyShort.Value, hfOrderID.Value, hfOrderDescription.Value, [Global].SecretKey)
                        hfHashDigest.Value = PaymentFormHelper.CalculateHashDigest(szStringToHash)
                    End If
                End If
            End If
        End Sub

        Private Function CheckIntegrityOfIncomingVariables(ByVal ivsIncomingVariableSource As INCOMING_VARIABLE_SOURCE) As Boolean
            Dim boReturnValue = False
            Dim szStringToHash As String

            Select Case ivsIncomingVariableSource
                Case INCOMING_VARIABLE_SOURCE.SHOPPING_CART_CHECKOUT
                    szStringToHash = PaymentFormHelper.GenerateStringToHash(Request.Form("hfShoppingCartAmount"), Request.Form("hfShoppingCartCurrencyShort"), Request.Form("hfShoppingCartOrderID"), Request.Form("hfShoppingCartOrderDescription"), [Global].SecretKey)

                    If Request.Form("hfShoppingCartHashDigest") = PaymentFormHelper.CalculateHashDigest(szStringToHash) Then
                        boReturnValue = True
                    End If
                Case INCOMING_VARIABLE_SOURCE.PAYMENT_FORM_POSTBACK
                    szStringToHash = PaymentFormHelper.GenerateStringToHash(hfAmount.Value, hfCurrencyShort.Value, hfOrderID.Value, hfOrderDescription.Value, [Global].SecretKey)

                    If hfHashDigest.Value = PaymentFormHelper.CalculateHashDigest(szStringToHash) Then
                        boReturnValue = True
                    End If
                Case INCOMING_VARIABLE_SOURCE.THREE_D_SECURE_FINGERPRINT
                    szStringToHash = PaymentFormHelper.GenerateStringToHash(Request.Form("hfThreeDSMethodData"), [Global].SecretKey)

                    If Request.Form("hfShoppingCartHashDigest") = PaymentFormHelper.CalculateHashDigest(szStringToHash) Then
                        boReturnValue = True
                    End If
                Case INCOMING_VARIABLE_SOURCE.THREE_D_SECURE_CHALLENGE
                    szStringToHash = PaymentFormHelper.GenerateStringToHash(Request.Form("hfCRES"), Request.Form("hfCrossReference"), [Global].SecretKey)

                    If Request.Form("hfShoppingCartHashDigest") = PaymentFormHelper.CalculateHashDigest(szStringToHash) Then
                        boReturnValue = True
                    End If

                Case Else
                    Throw New Exception("Invalid incoming variable source: " & ivsIncomingVariableSource.ToString())
            End Select

            Return boReturnValue
        End Function

        Protected Overrides Sub OnPreRender(ByVal e As EventArgs)
            MyBase.OnPreRender(e)

            ' need to work out what mode to put the form into
            Select Case m_fmFormMode
                Case FORM_MODE.PAYMENT_FORM
                    PaymentFormMultiView.SetActiveView(PaymentFormView)
                    BodyTag.Attributes("onload") = "gatherBrowserData();"
                Case FORM_MODE.THREE_D_SECURE_FINGERPRINT
                    PaymentFormMultiView.SetActiveView(ThreeDSecureView)

                    threeDSecureFrame.Attributes("src") = PaymentFormHelper.GetSiteSecureBaseURL(Request) & "Loading.htm"
                    BodyTag.Attributes("onload") = "document.threeDSecureEnvironmentForm.submit();"
                Case FORM_MODE.THREE_D_SECURE_CHALLENGE
                    PaymentFormMultiView.SetActiveView(ThreeDSecureView)

                    threeDSecureFrame.Attributes("src") = PaymentFormHelper.GetSiteSecureBaseURL(Request) & "Loading.htm"
                    BodyTag.Attributes("onload") = "document.threeDSecureChallengeForm.submit();"
                Case FORM_MODE.RESULTS
                    PaymentFormMultiView.SetActiveView(ResultsView)
                Case Else
                    Throw New Exception("Invalid form mode: " & m_fmFormMode.ToString())
            End Select

            hfFormMode.Value = m_fmFormMode.ToString()
        End Sub

        Private Sub PerformThreeDSecureAuthentication(ByVal szCRES As String, ByVal szCrossReference As String)
            Dim szMessage As String = Nothing
            Dim boTransactionSuccessful = False
            Dim sbString As StringBuilder
            Dim nCount = 0
            Dim tdsaThreeDSecureAuthentication As ThreeDSecureAuthentication
            Dim szPreviousTransactionMessage As String = Nothing
            Dim boDuplicateTransaction = False
            Dim szResponseCrossReference As String

            If m_boIncomingVariableErrors = True Then
                Return
            End If

            tdsaThreeDSecureAuthentication = New ThreeDSecureAuthentication()
            ' you need to put the correct gateway entry point urls in here
            ' contact support to get the correct urls

            ' The actual values to use for the entry points can be established in a number of ways
            ' 1) By periodically issuing a call to GetGatewayEntryPoints
            ' 2) By storing the values for the entry points returned with each transaction
            ' 3) Speculatively firing transactions at https://gw1.xxx followed by gw2, gw3, gw4....
            ' The lower the metric (2nd parameter) means that entry point will be attempted first,
            ' EXCEPT if it is -1 - in this case that entry point will be skipped
            ' NOTE: You do NOT have to add the entry points in any particular order - the list is sorted
            ' by metric value before the transaction sumbitting process begins
            ' The 3rd parameter is a retry attempt, so it is possible to try that entry point that number of times
            ' before failing over onto the next entry point in the list
            tdsaThreeDSecureAuthentication.RequestGatewayEntryPoints.Add("https://gw1." & [Global].PaymentProcessorFullDomain, 100, 1)
            tdsaThreeDSecureAuthentication.RequestGatewayEntryPoints.Add("https://gw2." & [Global].PaymentProcessorFullDomain, 200, 1)
            tdsaThreeDSecureAuthentication.RequestGatewayEntryPoints.Add("https://gw3." & [Global].PaymentProcessorFullDomain, 300, 1)

            tdsaThreeDSecureAuthentication.MerchantAuthentication.MerchantID = [Global].MerchantID
            tdsaThreeDSecureAuthentication.MerchantAuthentication.Password = [Global].Password

            tdsaThreeDSecureAuthentication.ThreeDSecureInputData.CrossReference = szCrossReference
            tdsaThreeDSecureAuthentication.ThreeDSecureInputData.CRES = szCRES

            ' send the SOAP request
            If Not tdsaThreeDSecureAuthentication.ProcessTransaction() Then
                szMessage = "Couldn't communicate with payment gateway"
                boTransactionSuccessful = True

                PaymentFormHelper.ReportTransactionResults(szCrossReference, 30, szMessage, Nothing)
            Else
                Select Case tdsaThreeDSecureAuthentication.ThreeDSecureAuthenticationResult.StatusCode
                    Case 0
                        ' status code of 0 - means transaction successful 
                        boTransactionSuccessful = True
                        m_fmFormMode = FORM_MODE.RESULTS
                        szMessage = tdsaThreeDSecureAuthentication.ThreeDSecureAuthenticationResult.Message
                        PaymentFormHelper.ReportTransactionResults(szCrossReference, tdsaThreeDSecureAuthentication.ThreeDSecureAuthenticationResult.StatusCode, tdsaThreeDSecureAuthentication.ThreeDSecureAuthenticationResult.Message, tdsaThreeDSecureAuthentication.TransactionOutputData.CrossReference)
                    Case 5
                        ' status code of 5 - means transaction declined 
                        boTransactionSuccessful = False
                        m_fmFormMode = FORM_MODE.RESULTS
                        szMessage = tdsaThreeDSecureAuthentication.ThreeDSecureAuthenticationResult.Message
                        PaymentFormHelper.ReportTransactionResults(szCrossReference, tdsaThreeDSecureAuthentication.ThreeDSecureAuthenticationResult.StatusCode, tdsaThreeDSecureAuthentication.ThreeDSecureAuthenticationResult.Message, tdsaThreeDSecureAuthentication.TransactionOutputData.CrossReference)
                    Case 20
                        ' status code of 20 - means duplicate transaction 
                        m_fmFormMode = FORM_MODE.RESULTS
                        szMessage = tdsaThreeDSecureAuthentication.ThreeDSecureAuthenticationResult.Message
                        If tdsaThreeDSecureAuthentication.ThreeDSecureAuthenticationResult.PreviousTransactionResult.StatusCode.Value = 0 Then
                            boTransactionSuccessful = True
                        Else
                            boTransactionSuccessful = False
                        End If
                        szPreviousTransactionMessage = tdsaThreeDSecureAuthentication.ThreeDSecureAuthenticationResult.PreviousTransactionResult.Message
                        boDuplicateTransaction = True

                        PaymentFormHelper.ReportTransactionResults(szCrossReference, tdsaThreeDSecureAuthentication.ThreeDSecureAuthenticationResult.PreviousTransactionResult.StatusCode.Value, tdsaThreeDSecureAuthentication.ThreeDSecureAuthenticationResult.PreviousTransactionResult.Message, tdsaThreeDSecureAuthentication.TransactionOutputData.CrossReference)
                    Case 30
                        ' status code of 30 - means an error occurred 
                        boTransactionSuccessful = False
                        m_fmFormMode = FORM_MODE.RESULTS

                        sbString = New StringBuilder()

                        ' get any additional messages
                        If tdsaThreeDSecureAuthentication.ThreeDSecureAuthenticationResult.ErrorMessages.Count > 0 Then
                            sbString.Append("<br /><ul>")

                            For nCount = 0 To tdsaThreeDSecureAuthentication.ThreeDSecureAuthenticationResult.ErrorMessages.Count - 1
                                sbString.AppendFormat("<li>{0}</li>", tdsaThreeDSecureAuthentication.ThreeDSecureAuthenticationResult.ErrorMessages(nCount))
                            Next
                            sbString.Append("</ul>")
                        End If

                        szMessage = tdsaThreeDSecureAuthentication.ThreeDSecureAuthenticationResult.Message & sbString.ToString()

                        If tdsaThreeDSecureAuthentication.TransactionOutputData Is Nothing Then
                            szResponseCrossReference = Nothing
                        Else
                            szResponseCrossReference = tdsaThreeDSecureAuthentication.TransactionOutputData.CrossReference
                        End If
                        PaymentFormHelper.ReportTransactionResults(szCrossReference, tdsaThreeDSecureAuthentication.ThreeDSecureAuthenticationResult.StatusCode, tdsaThreeDSecureAuthentication.ThreeDSecureAuthenticationResult.Message, szResponseCrossReference)
                    Case Else
                        ' unhandled status code  
                        boTransactionSuccessful = False
                        m_fmFormMode = FORM_MODE.RESULTS
                        szMessage = tdsaThreeDSecureAuthentication.ThreeDSecureAuthenticationResult.Message

                        If tdsaThreeDSecureAuthentication.TransactionOutputData Is Nothing Then
                            szResponseCrossReference = Nothing
                        Else
                            szResponseCrossReference = tdsaThreeDSecureAuthentication.TransactionOutputData.CrossReference
                        End If
                        PaymentFormHelper.ReportTransactionResults(szCrossReference, tdsaThreeDSecureAuthentication.ThreeDSecureAuthenticationResult.StatusCode, tdsaThreeDSecureAuthentication.ThreeDSecureAuthenticationResult.Message, szResponseCrossReference)
                End Select
            End If

            pnTransactionResultsPanel.Visible = True
            pnMessagePanel.Visible = False

            If Not boTransactionSuccessful Then
                pnTransactionResultsPanel.CssClass = "ErrorMessage"
            Else
                pnTransactionResultsPanel.CssClass = "SuccessMessage"
            End If

            lbGatewayResponse.Text = szMessage

            ' sort out the duplicate transaction reporting
            If boDuplicateTransaction Then
                pnDuplicateTransactionPanel.Visible = True
                lbPreviousTransactionMessage.Text = szPreviousTransactionMessage
            End If
        End Sub

        Private Sub PerformThreeDSecureEnvironment(ByVal szThreeDSMethodData As String, ByVal szCrossReference As String)
            Dim szMessage As String = Nothing
            Dim boTransactionSuccessful = False
            Dim sbString As StringBuilder
            Dim nCount = 0
            Dim tdseThreeDSecureEnvironment As ThreeDSecureEnvironment
            Dim szPreviousTransactionMessage As String = Nothing
            Dim boDuplicateTransaction = False
            Dim szResponseCrossReference As String

            If m_boIncomingVariableErrors = True Then
                Return
            End If

            tdseThreeDSecureEnvironment = New ThreeDSecureEnvironment()
            ' you need to put the correct gateway entry point urls in here
            ' contact support to get the correct urls

            ' The actual values to use for the entry points can be established in a number of ways
            ' 1) By periodically issuing a call to GetGatewayEntryPoints
            ' 2) By storing the values for the entry points returned with each transaction
            ' 3) Speculatively firing transactions at https://gw1.xxx followed by gw2, gw3, gw4....
            ' The lower the metric (2nd parameter) means that entry point will be attempted first,
            ' EXCEPT if it is -1 - in this case that entry point will be skipped
            ' NOTE: You do NOT have to add the entry points in any particular order - the list is sorted
            ' by metric value before the transaction sumbitting process begins
            ' The 3rd parameter is a retry attempt, so it is possible to try that entry point that number of times
            ' before failing over onto the next entry point in the list
            tdseThreeDSecureEnvironment.RequestGatewayEntryPoints.Add("https://gw1." & [Global].PaymentProcessorFullDomain, 100, 1)
            tdseThreeDSecureEnvironment.RequestGatewayEntryPoints.Add("https://gw2." & [Global].PaymentProcessorFullDomain, 200, 1)
            tdseThreeDSecureEnvironment.RequestGatewayEntryPoints.Add("https://gw3." & [Global].PaymentProcessorFullDomain, 300, 1)

            tdseThreeDSecureEnvironment.MerchantAuthentication.MerchantID = [Global].MerchantID
            tdseThreeDSecureEnvironment.MerchantAuthentication.Password = [Global].Password

            tdseThreeDSecureEnvironment.ThreeDSecureEnvironmentData.CrossReference = szCrossReference
            tdseThreeDSecureEnvironment.ThreeDSecureEnvironmentData.MethodData = szThreeDSMethodData

            ' send the SOAP request
            If Not tdseThreeDSecureEnvironment.ProcessTransaction() Then
                szMessage = "Couldn't communicate with payment gateway"
                boTransactionSuccessful = True

                PaymentFormHelper.ReportTransactionResults(szCrossReference, 30, szMessage, Nothing)
            Else
                Select Case tdseThreeDSecureEnvironment.ThreeDSecureEnvironmentResult.StatusCode
                    Case 0
                        ' status code of 0 - means transaction successful 
                        boTransactionSuccessful = True
                        m_fmFormMode = FORM_MODE.RESULTS
                        szMessage = tdseThreeDSecureEnvironment.ThreeDSecureEnvironmentResult.Message
                        PaymentFormHelper.ReportTransactionResults(szCrossReference, tdseThreeDSecureEnvironment.ThreeDSecureEnvironmentResult.StatusCode, tdseThreeDSecureEnvironment.ThreeDSecureEnvironmentResult.Message, tdseThreeDSecureEnvironment.TransactionOutputData.CrossReference)
                    Case 3
                        ' status code of 3 - means 3D Secure authentication required 
                        m_fmFormMode = FORM_MODE.THREE_D_SECURE_CHALLENGE

                        m_szACSURL = tdseThreeDSecureEnvironment.TransactionOutputData.ThreeDSecureOutputData.ACSURL
                        m_szCREQ = tdseThreeDSecureEnvironment.TransactionOutputData.ThreeDSecureOutputData.CREQ
                        m_szThreeDSSessionData = Request.Cookies("cr").Value
                        PaymentFormHelper.ReportTransactionResults(hfOrderID.Value, tdseThreeDSecureEnvironment.ThreeDSecureEnvironmentResult.StatusCode, tdseThreeDSecureEnvironment.ThreeDSecureEnvironmentResult.Message, tdseThreeDSecureEnvironment.TransactionOutputData.CrossReference)
                    Case 5
                        ' status code of 5 - means transaction declined 
                        boTransactionSuccessful = False
                        m_fmFormMode = FORM_MODE.RESULTS
                        szMessage = tdseThreeDSecureEnvironment.ThreeDSecureEnvironmentResult.Message
                        PaymentFormHelper.ReportTransactionResults(szCrossReference, tdseThreeDSecureEnvironment.ThreeDSecureEnvironmentResult.StatusCode, tdseThreeDSecureEnvironment.ThreeDSecureEnvironmentResult.Message, tdseThreeDSecureEnvironment.TransactionOutputData.CrossReference)
                    Case 20
                        ' status code of 20 - means duplicate transaction 
                        m_fmFormMode = FORM_MODE.RESULTS
                        szMessage = tdseThreeDSecureEnvironment.ThreeDSecureEnvironmentResult.Message
                        If tdseThreeDSecureEnvironment.ThreeDSecureEnvironmentResult.PreviousTransactionResult.StatusCode.Value = 0 Then
                            boTransactionSuccessful = True
                        Else
                            boTransactionSuccessful = False
                        End If
                        szPreviousTransactionMessage = tdseThreeDSecureEnvironment.ThreeDSecureEnvironmentResult.PreviousTransactionResult.Message
                        boDuplicateTransaction = True

                        PaymentFormHelper.ReportTransactionResults(szCrossReference, tdseThreeDSecureEnvironment.ThreeDSecureEnvironmentResult.PreviousTransactionResult.StatusCode.Value, tdseThreeDSecureEnvironment.ThreeDSecureEnvironmentResult.PreviousTransactionResult.Message, tdseThreeDSecureEnvironment.TransactionOutputData.CrossReference)
                    Case 30
                        ' status code of 30 - means an error occurred 
                        boTransactionSuccessful = False
                        m_fmFormMode = FORM_MODE.RESULTS

                        sbString = New StringBuilder()

                        ' get any additional messages
                        If tdseThreeDSecureEnvironment.ThreeDSecureEnvironmentResult.ErrorMessages.Count > 0 Then
                            sbString.Append("<br /><ul>")

                            For nCount = 0 To tdseThreeDSecureEnvironment.ThreeDSecureEnvironmentResult.ErrorMessages.Count - 1
                                sbString.AppendFormat("<li>{0}</li>", tdseThreeDSecureEnvironment.ThreeDSecureEnvironmentResult.ErrorMessages(nCount))
                            Next
                            sbString.Append("</ul>")
                        End If

                        szMessage = tdseThreeDSecureEnvironment.ThreeDSecureEnvironmentResult.Message & sbString.ToString()

                        If tdseThreeDSecureEnvironment.TransactionOutputData Is Nothing Then
                            szResponseCrossReference = Nothing
                        Else
                            szResponseCrossReference = tdseThreeDSecureEnvironment.TransactionOutputData.CrossReference
                        End If
                        PaymentFormHelper.ReportTransactionResults(szCrossReference, tdseThreeDSecureEnvironment.ThreeDSecureEnvironmentResult.StatusCode, tdseThreeDSecureEnvironment.ThreeDSecureEnvironmentResult.Message, szResponseCrossReference)
                    Case Else
                        ' unhandled status code  
                        boTransactionSuccessful = False
                        m_fmFormMode = FORM_MODE.RESULTS
                        szMessage = tdseThreeDSecureEnvironment.ThreeDSecureEnvironmentResult.Message

                        If tdseThreeDSecureEnvironment.TransactionOutputData Is Nothing Then
                            szResponseCrossReference = Nothing
                        Else
                            szResponseCrossReference = tdseThreeDSecureEnvironment.TransactionOutputData.CrossReference
                        End If
                        PaymentFormHelper.ReportTransactionResults(szCrossReference, tdseThreeDSecureEnvironment.ThreeDSecureEnvironmentResult.StatusCode, tdseThreeDSecureEnvironment.ThreeDSecureEnvironmentResult.Message, szResponseCrossReference)
                End Select
            End If

            pnTransactionResultsPanel.Visible = True
            pnMessagePanel.Visible = False

            If Not boTransactionSuccessful Then
                pnTransactionResultsPanel.CssClass = "ErrorMessage"
            Else
                pnTransactionResultsPanel.CssClass = "SuccessMessage"
            End If

            lbGatewayResponse.Text = szMessage

            ' sort out the duplicate transaction reporting
            If boDuplicateTransaction Then
                pnDuplicateTransactionPanel.Visible = True
                lbPreviousTransactionMessage.Text = szPreviousTransactionMessage
            End If
        End Sub

        Private Sub PopulateFormItems()
            Dim szWorkingValue As String
            Dim szWorkingDisplayValue As String
            Dim nCount = 0
            Dim nCurrentYear As Integer
            Dim liListItem As ListItem
            Dim boHitFirstZeroPriorityCountry = False
            Dim icISOCurrency As ThePaymentGateway.Common.ISOCurrency

            If m_boIncomingVariableErrors = True Then
                Return
            End If

            ' read the values from the shopping cart checkout
            hfAmount.Value = Request.Form("hfShoppingCartAmount")
            hfCurrencyShort.Value = Request.Form("hfShoppingCartCurrencyShort")
            hfOrderID.Value = Request.Form("hfShoppingCartOrderID")
            hfOrderDescription.Value = Request.Form("hfShoppingCartOrderDescription")

            ' populate the label text
            If Not [Global].ISOCurrencyData.GetISOCurrency(hfCurrencyShort.Value, icISOCurrency) Then
                Throw New Exception("Couldn't get ISO currency: " & hfCurrencyShort.Value)
            End If
            lbAmount.Text = icISOCurrency.GetAmountCurrencyString(Convert.ToInt32(Request.Form("hfShoppingCartAmount")))
            lbOrderID.Text = Request.Form("hfShoppingCartOrderID")
            lbOrderDescription.Text = Request.Form("hfShoppingCartOrderDescription")

            tbAddress1.Text = ""
            tbAddress2.Text = ""
            tbAddress3.Text = ""
            tbAddress4.Text = ""
            tbCity.Text = ""
            tbState.Text = ""
            tbPostCode.Text = ""
            tbCardName.Text = ""
            tbCardNumber.Text = ""
            tbIssueNumber.Text = ""
            tbCV2.Text = ""
            lbMessageLabel.Text = ""
            pnMessagePanel.Visible = False

            liListItem = New ListItem("", "-1")
            ddlCountryShort.Items.Add(liListItem)

            ' populate the country drop down
            For nCount = 0 To [Global].ISOCountryData.Count - 1
                If nCount > 0 AndAlso Not boHitFirstZeroPriorityCountry AndAlso [Global].ISOCountryData(nCount).ListPriority = 0 Then
                    boHitFirstZeroPriorityCountry = True
                    liListItem = New ListItem("---------------------", "-1")
                    ddlCountryShort.Items.Add(liListItem)
                End If
                liListItem = New ListItem([Global].ISOCountryData(nCount).CountryName, [Global].ISOCountryData(nCount).CountryShort)

                ddlCountryShort.Items.Add(liListItem)
            Next

            nCurrentYear = Date.Now.Year

            ddExpiryDateYear.Items.Add("")
            ddExpiryDateMonth.Items.Add("")

            ddStartDateYear.Items.Add("")
            ddStartDateMonth.Items.Add("")

            For nCount = 0 To 9
                ' expiry date - start with this year & add 10
                szWorkingDisplayValue = Convert.ToString(nCurrentYear + nCount)
                szWorkingValue = szWorkingDisplayValue.Substring(2)
                liListItem = New ListItem(szWorkingDisplayValue, szWorkingValue)
                ddExpiryDateYear.Items.Add(liListItem)

                ' start date - start with this year & minus 10
                szWorkingDisplayValue = Convert.ToString(nCurrentYear - nCount)
                szWorkingValue = szWorkingDisplayValue.Substring(2)
                liListItem = New ListItem(szWorkingDisplayValue, szWorkingValue)
                ddStartDateYear.Items.Add(liListItem)
            Next

            For nCount = 1 To 12
                szWorkingValue = Convert.ToString(nCount)
                szWorkingDisplayValue = MyCompanyName.PaymentFormHelper.ForwardPaddedNumberString(nCount, 2, "0"c)
                liListItem = New ListItem(szWorkingDisplayValue, szWorkingValue)
                ddExpiryDateMonth.Items.Add(liListItem)

                liListItem = New ListItem(szWorkingDisplayValue, szWorkingValue)
                ddStartDateMonth.Items.Add(liListItem)
            Next
        End Sub

        Protected Sub SubmitButton_Click(ByVal sender As Object, ByVal e As EventArgs)
            Dim szMessage As String = Nothing
            Dim boTransactionSuccessful = False
            Dim sbString As StringBuilder
            Dim nCount = 0
            Dim cdtCardDetailsTransaction As CardDetailsTransaction
            Dim szPreviousTransactionMessage As String = Nothing
            Dim boDuplicateTransaction = False
            Dim szCrossReference As String
            Dim icISOCountry As ISOCountry
            Dim icISOCurrency As ISOCurrency

            If m_boIncomingVariableErrors = True Then
                Return
            End If

            cdtCardDetailsTransaction = New CardDetailsTransaction()
            ' you need to put the correct gateway entry point urls in here
            ' contact support to get the correct urls

            ' The actual values to use for the entry points can be established in a number of ways
            ' 1) By periodically issuing a call to GetGatewayEntryPoints
            ' 2) By storing the values for the entry points returned with each transaction
            ' 3) Speculatively firing transactions at https://gw1.xxx followed by gw2, gw3, gw4....
            ' The lower the metric (2nd parameter) means that entry point will be attempted first,
            ' EXCEPT if it is -1 - in this case that entry point will be skipped
            ' NOTE: You do NOT have to add the entry points in any particular order - the list is sorted
            ' by metric value before the transaction sumbitting process begins
            ' The 3rd parameter is a retry attempt, so it is possible to try that entry point that number of times
            ' before failing over onto the next entry point in the list
            cdtCardDetailsTransaction.RequestGatewayEntryPoints.Add("https://gw1." & [Global].PaymentProcessorFullDomain, 100, 1)
            cdtCardDetailsTransaction.RequestGatewayEntryPoints.Add("https://gw2." & [Global].PaymentProcessorFullDomain, 200, 1)
            cdtCardDetailsTransaction.RequestGatewayEntryPoints.Add("https://gw3." & [Global].PaymentProcessorFullDomain, 300, 1)

            cdtCardDetailsTransaction.MerchantAuthentication.MerchantID = [Global].MerchantID
            cdtCardDetailsTransaction.MerchantAuthentication.Password = [Global].Password

            cdtCardDetailsTransaction.TransactionDetails.MessageDetails.TransactionType = "SALE"

            cdtCardDetailsTransaction.TransactionDetails.Amount.Value = Convert.ToInt32(hfAmount.Value)
            If Not String.IsNullOrEmpty(hfCurrencyShort.Value) Then
                If [Global].ISOCurrencyData.GetISOCurrency(hfCurrencyShort.Value, icISOCurrency) Then
                    cdtCardDetailsTransaction.TransactionDetails.CurrencyCode.Value = icISOCurrency.ISOCode
                End If
            End If
            cdtCardDetailsTransaction.TransactionDetails.OrderID = hfOrderID.Value
            cdtCardDetailsTransaction.TransactionDetails.OrderDescription = hfOrderDescription.Value

            cdtCardDetailsTransaction.CardDetails.CardName = tbCardName.Text
            cdtCardDetailsTransaction.CardDetails.CardNumber = tbCardNumber.Text
            If Not String.IsNullOrEmpty(ddExpiryDateMonth.SelectedValue) Then
                cdtCardDetailsTransaction.CardDetails.ExpiryDate.Month.Value = Convert.ToInt32(ddExpiryDateMonth.SelectedValue)
            End If
            If Not String.IsNullOrEmpty(ddExpiryDateYear.SelectedValue) Then
                cdtCardDetailsTransaction.CardDetails.ExpiryDate.Year.Value = Convert.ToInt32(ddExpiryDateYear.SelectedValue)
            End If
            If Not String.IsNullOrEmpty(ddStartDateMonth.SelectedValue) Then
                cdtCardDetailsTransaction.CardDetails.StartDate.Month.Value = Convert.ToInt32(ddStartDateMonth.SelectedValue)
            End If
            If Not String.IsNullOrEmpty(ddStartDateYear.SelectedValue) Then
                cdtCardDetailsTransaction.CardDetails.ExpiryDate.Year.Value = Convert.ToInt32(ddStartDateYear.SelectedValue)
            End If
            cdtCardDetailsTransaction.CardDetails.IssueNumber = tbIssueNumber.Text
            cdtCardDetailsTransaction.CardDetails.CV2 = tbCV2.Text

            cdtCardDetailsTransaction.CustomerDetails.EmailAddress = "test@test.com"
            cdtCardDetailsTransaction.CustomerDetails.PhoneNumber = "123456789"
            ' the direct/API can only currently handle IPv4 addresses.
            ' Uncomment the following line out if the UserHostAddress will carry an IPv4 address
            'cdtCardDetailsTransaction.CustomerDetails.CustomerIPAddress = Request.UserHostAddress;
            cdtCardDetailsTransaction.CustomerDetails.BillingAddress.Address1 = tbAddress1.Text
            cdtCardDetailsTransaction.CustomerDetails.BillingAddress.Address2 = tbAddress2.Text
            cdtCardDetailsTransaction.CustomerDetails.BillingAddress.Address3 = tbAddress3.Text
            cdtCardDetailsTransaction.CustomerDetails.BillingAddress.Address4 = tbAddress4.Text
            cdtCardDetailsTransaction.CustomerDetails.BillingAddress.City = tbCity.Text
            cdtCardDetailsTransaction.CustomerDetails.BillingAddress.State = tbState.Text
            cdtCardDetailsTransaction.CustomerDetails.BillingAddress.PostCode = tbPostCode.Text
            If Not String.IsNullOrEmpty(ddlCountryShort.SelectedValue) AndAlso ddlCountryShort.SelectedValue IsNot "-1" Then
                If [Global].ISOCountryData.GetISOCountry(ddlCountryShort.SelectedValue, icISOCountry) Then
                    cdtCardDetailsTransaction.CustomerDetails.BillingAddress.CountryCode.Value = icISOCountry.ISOCode
                End If
            End If

            cdtCardDetailsTransaction.TransactionDetails.TransactionControl.EchoCardType.Value = True
            cdtCardDetailsTransaction.TransactionDetails.TransactionControl.EchoAVSCheckResult.Value = True
            cdtCardDetailsTransaction.TransactionDetails.TransactionControl.EchoCV2CheckResult.Value = True
            cdtCardDetailsTransaction.TransactionDetails.TransactionControl.DuplicateDelay.Value = 60
            cdtCardDetailsTransaction.TransactionDetails.TransactionControl.ThreeDSecureOverridePolicy.Value = True

            cdtCardDetailsTransaction.TransactionDetails.ThreeDSecureBrowserDetails.DeviceCategory.Value = 0
            cdtCardDetailsTransaction.TransactionDetails.ThreeDSecureBrowserDetails.AcceptHeaders = "*/*"
            cdtCardDetailsTransaction.TransactionDetails.ThreeDSecureBrowserDetails.UserAgent = Request.UserAgent

            If Not String.IsNullOrEmpty(hfJavaEnabled.Value) Then
                cdtCardDetailsTransaction.TransactionDetails.ThreeDSecureBrowserDetails.JavaEnabled.Value = Convert.ToBoolean(hfJavaEnabled.Value)
            End If
            If Not String.IsNullOrEmpty(hfJavaScriptEnabled.Value) Then
                cdtCardDetailsTransaction.TransactionDetails.ThreeDSecureBrowserDetails.JavaScriptEnabled.Value = Convert.ToBoolean(hfJavaScriptEnabled.Value)
            End If
            If Not String.IsNullOrEmpty(hfScreenWidth.Value) Then
                cdtCardDetailsTransaction.TransactionDetails.ThreeDSecureBrowserDetails.ScreenWidth.Value = Convert.ToInt32(hfScreenWidth.Value)
            End If
            If Not String.IsNullOrEmpty(hfScreenHeight.Value) Then
                cdtCardDetailsTransaction.TransactionDetails.ThreeDSecureBrowserDetails.ScreenHeight.Value = Convert.ToInt32(hfScreenHeight.Value)
            End If
            If Not String.IsNullOrEmpty(hfScreenColourDepth.Value) Then
                cdtCardDetailsTransaction.TransactionDetails.ThreeDSecureBrowserDetails.ScreenColourDepth.Value = Convert.ToInt32(hfScreenColourDepth.Value)
            End If
            If Not String.IsNullOrEmpty(hfTimezoneOffset.Value) Then
                cdtCardDetailsTransaction.TransactionDetails.ThreeDSecureBrowserDetails.TimeZone.Value = Convert.ToInt32(hfTimezoneOffset.Value)
            End If
            cdtCardDetailsTransaction.TransactionDetails.ThreeDSecureBrowserDetails.Language = hfLanguage.Value

            cdtCardDetailsTransaction.TransactionDetails.ThreeDSecureNotificationDetails.ChallengeNotificationURL = PaymentFormHelper.GetSiteSecureBaseURL(Request) & "ThreeDSecureLandingPage.aspx"
            cdtCardDetailsTransaction.TransactionDetails.ThreeDSecureNotificationDetails.FingerprintNotificationURL = PaymentFormHelper.GetSiteSecureBaseURL(Request) & "ThreeDSecureLandingPage.aspx"

            ' send the SOAP request
            If Not cdtCardDetailsTransaction.ProcessTransaction() Then
                szMessage = "Couldn't communicate with payment gateway"
                boTransactionSuccessful = False

                PaymentFormHelper.ReportTransactionResults(hfOrderID.Value, 30, szMessage, Nothing)
            Else
                Select Case cdtCardDetailsTransaction.CardDetailsTransactionResult.StatusCode
                    Case 0
                        ' status code of 0 - means transaction successful 
                        boTransactionSuccessful = True
                        m_fmFormMode = FORM_MODE.RESULTS
                        szMessage = cdtCardDetailsTransaction.CardDetailsTransactionResult.Message
                        PaymentFormHelper.ReportTransactionResults(hfOrderID.Value, cdtCardDetailsTransaction.CardDetailsTransactionResult.StatusCode, cdtCardDetailsTransaction.CardDetailsTransactionResult.Message, cdtCardDetailsTransaction.TransactionOutputData.CrossReference)
                    Case 3
                        ' status code of 3 - means 3D Secure authentication required 
                        m_fmFormMode = FORM_MODE.THREE_D_SECURE_FINGERPRINT

                        m_szMethodDataURL = cdtCardDetailsTransaction.TransactionOutputData.ThreeDSecureOutputData.MethodURL
                        m_szThreeDSMethodData = cdtCardDetailsTransaction.TransactionOutputData.ThreeDSecureOutputData.MethodData
                        ' the cross reference needs to be stored somewhere where it can be retrieved again later. For this stage, the 3DSv2 spec doesn't have a "pass through" field
                        ' that just echoes back what it's given (unlike a later stage with the threeDSSessionData)
                        ' for now - just store this in a cookie
                        Response.Cookies.Add(New HttpCookie("cr", cdtCardDetailsTransaction.TransactionOutputData.CrossReference))
                        PaymentFormHelper.ReportTransactionResults(hfOrderID.Value, cdtCardDetailsTransaction.CardDetailsTransactionResult.StatusCode, cdtCardDetailsTransaction.CardDetailsTransactionResult.Message, cdtCardDetailsTransaction.TransactionOutputData.CrossReference)
                    Case 5
                        ' status code of 5 - means transaction declined 
                        boTransactionSuccessful = False
                        m_fmFormMode = FORM_MODE.RESULTS
                        szMessage = cdtCardDetailsTransaction.CardDetailsTransactionResult.Message
                        PaymentFormHelper.ReportTransactionResults(hfOrderID.Value, cdtCardDetailsTransaction.CardDetailsTransactionResult.StatusCode, cdtCardDetailsTransaction.CardDetailsTransactionResult.Message, cdtCardDetailsTransaction.TransactionOutputData.CrossReference)
                    Case 20
                        ' status code of 20 - means duplicate transaction 
                        m_fmFormMode = FORM_MODE.RESULTS
                        szMessage = cdtCardDetailsTransaction.CardDetailsTransactionResult.Message
                        If cdtCardDetailsTransaction.CardDetailsTransactionResult.PreviousTransactionResult.StatusCode.Value = 0 Then
                            boTransactionSuccessful = True
                        Else
                            boTransactionSuccessful = False
                        End If
                        szPreviousTransactionMessage = cdtCardDetailsTransaction.CardDetailsTransactionResult.PreviousTransactionResult.Message
                        boDuplicateTransaction = True
                        PaymentFormHelper.ReportTransactionResults(hfOrderID.Value, cdtCardDetailsTransaction.CardDetailsTransactionResult.PreviousTransactionResult.StatusCode.Value, szPreviousTransactionMessage, cdtCardDetailsTransaction.TransactionOutputData.CrossReference)
                    Case 30
                        ' status code of 30 - means an error occurred 
                        boTransactionSuccessful = False
                        m_fmFormMode = FORM_MODE.PAYMENT_FORM

                        sbString = New StringBuilder()

                        ' get any additional messages
                        If cdtCardDetailsTransaction.CardDetailsTransactionResult.ErrorMessages.Count > 0 Then
                            sbString.Append("<br /><ul>")

                            For nCount = 0 To cdtCardDetailsTransaction.CardDetailsTransactionResult.ErrorMessages.Count - 1
                                sbString.AppendFormat("<li>{0}</li>", cdtCardDetailsTransaction.CardDetailsTransactionResult.ErrorMessages(nCount))
                            Next
                            sbString.Append("</ul>")
                        End If

                        szMessage = cdtCardDetailsTransaction.CardDetailsTransactionResult.Message & sbString.ToString()

                        If cdtCardDetailsTransaction.TransactionOutputData Is Nothing Then
                            szCrossReference = Nothing
                        Else
                            szCrossReference = cdtCardDetailsTransaction.TransactionOutputData.CrossReference
                        End If

                        PaymentFormHelper.ReportTransactionResults(hfOrderID.Value, cdtCardDetailsTransaction.CardDetailsTransactionResult.StatusCode, cdtCardDetailsTransaction.CardDetailsTransactionResult.Message, szCrossReference)
                    Case Else
                        ' unhandled status code  
                        boTransactionSuccessful = False
                        m_fmFormMode = FORM_MODE.PAYMENT_FORM
                        szMessage = cdtCardDetailsTransaction.CardDetailsTransactionResult.Message

                        If cdtCardDetailsTransaction.TransactionOutputData Is Nothing Then
                            szCrossReference = Nothing
                        Else
                            szCrossReference = cdtCardDetailsTransaction.TransactionOutputData.CrossReference
                        End If

                        PaymentFormHelper.ReportTransactionResults(hfOrderID.Value, cdtCardDetailsTransaction.CardDetailsTransactionResult.StatusCode, cdtCardDetailsTransaction.CardDetailsTransactionResult.Message, szCrossReference)
                End Select
            End If

            If m_fmFormMode = FORM_MODE.PAYMENT_FORM Then
                pnMessagePanel.CssClass = "ErrorMessage"
                pnMessagePanel.Visible = True
                lbMessageLabel.Text = szMessage
                pnTransactionResultsPanel.Visible = False
            Else
                pnTransactionResultsPanel.Visible = True
                pnMessagePanel.Visible = False

                If Not boTransactionSuccessful Then
                    pnTransactionResultsPanel.CssClass = "ErrorMessage"
                Else
                    pnTransactionResultsPanel.CssClass = "SuccessMessage"
                End If

                lbGatewayResponse.Text = szMessage

                ' sort out the duplicate transaction reporting
                If boDuplicateTransaction Then
                    pnDuplicateTransactionPanel.Visible = True
                    lbPreviousTransactionMessage.Text = szPreviousTransactionMessage
                End If
            End If
        End Sub
    End Class
End Namespace
