<cffunction name="IncludeTemplate">
	<cfargument name="templateName" type="string" required="true">

	<cfinclude template="#templateName#">
</cffunction>
<cffunction name="Abort">
	<cfargument name="output" type="string" required="false">
	<cfif isDefined("output") EQ true>
		<cfoutput>#output#</cfoutput>
	</cfif>
	<cfabort>
</cffunction>
<cffunction name="Dump">
	<cfargument name="var" type="any" required="true">
	<cfdump var="#var#">
</cffunction>
<cffunction name="HTTPPost" returnType="string">
	<cfargument name="url" type="string" required="true">
	<cfargument name="postVariables" type="array" required="true">
    
    <cfset returnString="">

    <cfif ArrayLen(postVariables) GT 0>
    	<cfhttp url="#url#" method="post">
            <cfloop index="nLoopIndex" from="1" to="#ArrayLen(postVariables)#">
                <cfhttpparam type="formfield" name="#postVariables[nLoopIndex].Name#" value="#postVariables[nLoopIndex].Value#">
            </cfloop>
        </cfhttp>

        <cfset returnString=cfhttp.fileContent>
    </cfif>
    <cfreturn returnString>
</cffunction>
<cffunction name="Throw">
	<cfargument name="message" type="string" required="true">

	<cfthrow type="any" message="#message#">
</cffunction>

<cfscript>
	function CalculateHashDigest(szInputString, szPreSharedKey, szHashMethod)
    {
		szCalculatedHashDigest = "";

		switch (szHashMethod)
		{
			case "MD5":
        		szCalculatedHashDigest = Hash(szInputString, "MD5");
				break;
			case "SHA1":
        		szCalculatedHashDigest = Hash(szInputString, "SHA1");
				break;
			case "HMACMD5":
			case "HMACSHA1":
        		Throw("Hash method [" & szHashMethod & "] not supported with this version of ColdFusion [" & SERVER.ColdFusion.ProductVersion & "]");
		}

        return (szCalculatedHashDigest);
    }
	function GetTimeZoneOffset()
	{
		szReturnString = "";
		szPlusMinusSign = "+";
		szHourOffset = "00";
		szMinuteOffset = "00";

		iInfo = GetTimeZoneInfo();
		nTotalOffset = iInfo.utcTotalOffset;

		if (nTotalOffset LT 0)
		{
			szPlusMinusSign = "-";
			nTotalOffset = nTotalOffset * -1;
		}

		nMinuteOffset = nTotalOffset Mod 60;
		nHourOffset = (nTotalOffset - nMinuteOffset) / 60;

		if (nHourOffset LT 10)
		{
			szHourOffset = "0" & nHourOffset;
		}
		else
		{
			szHourOffset = nHourOffset;
		}

		if (nMinuteOffset LT 10)
		{
			szMinuteOffset = "0" & nMinuteOffset;
		}
		else
		{
			szMinuteOffset = nMinuteOffset;
		}

		szReturnString = szPlusMinusSign & szHourOffset & ":" & szMinuteOffset;

		return (szReturnString);
	}
	function GetSiteSecureBaseURL()
	{
		szReturnString = "";
		szPortString = "";
		szProtocolString = "";

		if (CGI.HTTPS EQ "on")
		{
			szProtocolString = "https://";
			if (CGI.SERVER_PORT NEQ 443)
			{
				szPortString = ":" & CGI.SERVER_PORT;
			}
		}
		else
		{
			szProtocolString = "http://";			
			if (CGI.SERVER_PORT NEQ 80)
			{
				szPortString = ":" & CGI.SERVER_PORT;
			}
		}

		if(Len(CGI.PATH_INFO) GT 0)
		{
			szReturnString = szProtocolString & CGI.SERVER_NAME & szPortString & CGI.PATH_INFO;
		}
		else
		{
			szReturnString = szProtocolString & CGI.SERVER_NAME & szPortString & CGI.SCRIPT_NAME;
		}

		boFinished = false;
		nLoopIndex = Len(szReturnString);

		while (boFinished EQ false AND
			   nLoopIndex GTE 1)
		{
			if (Mid(szReturnString, nLoopIndex, 1) EQ "/")
			{
				boFinished = true;
				szReturnString = Mid(szReturnString, 1, nLoopIndex);
			}
			nLoopIndex = nLoopIndex - 1;
		}

		return (szReturnString);
	}

	function GetTransactionReferenceFromQueryString(aQueryStringVariables, szCrossReference, szOrderID, szHashDigest, szOutputMessage)
	{
		boErrorOccurred = false;
		boReturnValue = false;	
		szCrossReference.Value = "";
		szOrderID.Value = "";		
		szHashDigest.Value = "";
		szOutputMessage.Value = "";

		try
		{
			// hash digest
			if (isDefined("aQueryStringVariables.HashDigest") EQ true)
			{
				szHashDigest.Value = aQueryStringVariables.HashDigest;
			}

			// cross reference of transaction
			if (isDefined("aQueryStringVariables.CrossReference") NEQ true)
			{
				szOutputMessage.Value = AddStringToStringList(szOutputMessage.Value, "Expected variable [CrossReference] not received");
				boErrorOccurred = true;
			}
			else
			{
				szCrossReference.Value = aQueryStringVariables.CrossReference;
			}
			// order ID (same as value passed into payment form - echoed back out by payment form)
			if (isDefined("aQueryStringVariables.OrderID") NEQ true)
			{
				szOutputMessage.Value = AddStringToStringList(szOutputMessage.Value, "Expected variable [OrderID] not received");
				boErrorOccurred = true;
			}
			else
			{
				szOrderID.Value = aQueryStringVariables.OrderID;
			}
		}
		catch (Any e)
		{
			boErrorOccurred = true;
			szOutputMessage.Value = e.message;
		}

		if (boErrorOccurred EQ true)
		{
			boReturnValue = false;
		}
		else
		{
			boReturnValue = true;
		}

		return (boReturnValue);
	}
	function GetTransactionResultFromPostVariables(aFormVariables, trTransactionResult, szHashDigest, szOutputMessage)
	{
		boErrorOccurred = false;
		boReturnValue = false;
		szOutputMessage.Value = "";
		szHashDigest.Value = "";

		try
		{
			// hash digest
			if (isDefined("aFormVariables.HashDigest") EQ true)
			{
				szHashDigest.Value = aFormVariables.HashDigest;
			}

			// transaction status code
			if (isDefined("aFormVariables.StatusCode") NEQ true)
			{
				szOutputMessage.Value = AddStringToStringList(szOutputMessage.Value, "Expected variable [StatusCode] not received");
				boErrorOccurred = true;
			}
			else
			{
				if (aFormVariables.StatusCode EQ "")
				{
					nStatusCode = "";
				}
				else
				{
					nStatusCode = aFormVariables.StatusCode;
				}
			}
			// transaction message
			if (isDefined("aFormVariables.Message") NEQ true)
			{
				szOutputMessage.Value = AddStringToStringList(szOutputMessage.Value, "Expected variable [Message] not received");
				boErrorOccurred = true;
			}
			else
			{
				szMessage = aFormVariables.Message;
			}
			// status code of original transaction if this transaction was deemed a duplicate
			if (isDefined("aFormVariables.PreviousStatusCode") NEQ true)
			{
				szOutputMessage.Value = AddStringToStringList(szOutputMessage.Value, "Expected variable [PreviousStatusCode] not received");
				boErrorOccurred = true;
			}
			else
			{
				if (aFormVariables.PreviousStatusCode EQ "")
				{
					nPreviousStatusCode = "";
				}
				else
				{
					nPreviousStatusCode = aFormVariables.PreviousStatusCode;
				}
			}
			// status code of original transaction if this transaction was deemed a duplicate
			if (isDefined("aFormVariables.PreviousMessage") NEQ true)
			{
				szOutputMessage.Value = AddStringToStringList(szOutputMessage.Value, "Expected variable [PreviousMessage] not received");
				boErrorOccurred = true;
			}
			else
			{
				szPreviousMessage = aFormVariables.PreviousMessage;
			}
			// cross reference of transaction
			if (isDefined("aFormVariables.CrossReference") NEQ true)
			{
				szOutputMessage.Value = AddStringToStringList(szOutputMessage.Value, "Expected variable [CrossReference] not received");
				boErrorOccurred = true;
			}
			else
			{
				szCrossReference = aFormVariables.CrossReference;
			}
			// amount (same as value passed into payment form - echoed back out by payment form)
			if (isDefined("aFormVariables.Amount") NEQ true)
			{
				szOutputMessage.Value = AddStringToStringList(szOutputMessage.Value, "Expected variable [Amount] not received");
				boErrorOccurred = true;
			}
			else
			{
				if (aFormVariables.Amount EQ "")
				{
					nAmount = "";
				}
				else
				{
					nAmount = aFormVariables.Amount;
				}
			}
			// currency code (same as value passed into payment form - echoed back out by payment form)
			if (isDefined("aFormVariables.CurrencyCode") NEQ true)
			{
				szOutputMessage.Value = AddStringToStringList(szOutputMessage.Value, "Expected variable [CurrencyCode] not received");
				boErrorOccurred = true;
			}
			else
			{
				if (aFormVariables.CurrencyCode EQ "")
				{
					nCurrencyCode = "";
				}
				else
				{
					nCurrencyCode = aFormVariables.CurrencyCode;
				}
			}
			// order ID (same as value passed into payment form - echoed back out by payment form)
			if (isDefined("aFormVariables.OrderID") NEQ true)
			{
				szOutputMessage.Value = AddStringToStringList(szOutputMessage.Value, "Expected variable [OrderID] not received");
				boErrorOccurred = true;
			}
			else
			{
				szOrderID = aFormVariables.OrderID;
			}
			// transaction type (same as value passed into payment form - echoed back out by payment form)
			if (isDefined("aFormVariables.TransactionType") NEQ true)
			{
				szOutputMessage.Value = AddStringToStringList(szOutputMessage.Value, "Expected variable [TransactionType] not received");
				boErrorOccurred = true;
			}
			else
			{
				szTransactionType = aFormVariables.TransactionType;
			}
			// transaction date/time (same as value passed into payment form - echoed back out by payment form)
			if (isDefined("aFormVariables.TransactionDateTime") NEQ true)
			{
				szOutputMessage.Value = AddStringToStringList(szOutputMessage.Value, "Expected variable [TransactionDateTime] not received");
				boErrorOccurred = true;
			}
			else
			{
				szTransactionDateTime = aFormVariables.TransactionDateTime;
			}
			// order description (same as value passed into payment form - echoed back out by payment form)
			if (isDefined("aFormVariables.OrderDescription") NEQ true)
			{
				szOutputMessage.Value = AddStringToStringList(szOutputMessage.Value, "Expected variable [OrderDescription] not received");
				boErrorOccurred = true;
			}
			else
			{
				szOrderDescription = aFormVariables.OrderDescription;
			}
			// customer name (not necessarily the same as value passed into payment form - as the customer can change it on the form)
			if (isDefined("aFormVariables.CustomerName") NEQ true)
			{
				szOutputMessage.Value = AddStringToStringList(szOutputMessage.Value, "Expected variable [CustomerName] not received");
				boErrorOccurred = true;
			}
			else
			{
				szCustomerName = aFormVariables.CustomerName;
			}
			// address1 (not necessarily the same as value passed into payment form - as the customer can change it on the form)
			if (isDefined("aFormVariables.Address1") NEQ true)
			{
				szOutputMessage.Value = AddStringToStringList(szOutputMessage.Value, "Expected variable [Address1] not received");
				boErrorOccurred = true;
			}
			else
			{
				szAddress1 = aFormVariables.Address1;
			}
			// address2 (not necessarily the same as value passed into payment form - as the customer can change it on the form)
			if (isDefined("aFormVariables.Address2") NEQ true)
			{
				szOutputMessage.Value = AddStringToStringList(szOutputMessage.Value, "Expected variable [Address2] not received");
				boErrorOccurred = true;
			}
			else
			{
				szAddress2 = aFormVariables.Address2;
			}
			// address3 (not necessarily the same as value passed into payment form - as the customer can change it on the form)
			if (isDefined("aFormVariables.Address3") NEQ true)
			{
				szOutputMessage.Value = AddStringToStringList(szOutputMessage.Value, "Expected variable [Address3] not received");
				boErrorOccurred = true;
			}
			else
			{
				szAddress3 = aFormVariables.Address3;
			}
			// address4 (not necessarily the same as value passed into payment form - as the customer can change it on the form)
			if (isDefined("aFormVariables.Address4") NEQ true)
			{
				szOutputMessage.Value = AddStringToStringList(szOutputMessage.Value, "Expected variable [Address4] not received");
				boErrorOccurred = true;
			}
			else
			{
				szAddress4 = aFormVariables.Address4;
			}
			// city (not necessarily the same as value passed into payment form - as the customer can change it on the form)
			if (isDefined("aFormVariables.City") NEQ true)
			{
				szOutputMessage.Value = AddStringToStringList(szOutputMessage.Value, "Expected variable [City] not received");
				boErrorOccurred = true;
			}
			else
			{
				szCity = aFormVariables.City;
			}
			// state (not necessarily the same as value passed into payment form - as the customer can change it on the form)
			if (isDefined("aFormVariables.State") NEQ true)
			{
				szOutputMessage.Value = AddStringToStringList(szOutputMessage.Value, "Expected variable [State] not received");
				boErrorOccurred = true;
			}
			else
			{
				szState = aFormVariables.State;
			}
			// post code (not necessarily the same as value passed into payment form - as the customer can change it on the form)
			if (isDefined("aFormVariables.PostCode") NEQ true)
			{
				szOutputMessage.Value = AddStringToStringList(szOutputMessage.Value, "Expected variable [PostCode] not received");
				boErrorOccurred = true;
			}
			else
			{
				szPostCode = aFormVariables.PostCode;
			}
			// country code (not necessarily the same as value passed into payment form - as the customer can change it on the form)
			if (isDefined("aFormVariables.CountryCode") NEQ true)
			{
				szOutputMessage.Value = AddStringToStringList(szOutputMessage.Value, "Expected variable [CountryCode] not received");
				boErrorOccurred = true;
			}
			else
			{
				if (aFormVariables.CountryCode EQ "")
				{
					nCountryCode = "";
				}
				else
				{
					nCountryCode = aFormVariables.CountryCode;
				}
			}

			if (boErrorOccurred EQ false)
			{
				trTransactionResult.StatusCode = nStatusCode; // transaction status code
				trTransactionResult.Message = szMessage; // transaction message
				trTransactionResult.PreviousStatusCode = nPreviousStatusCode; // status code of original transaction if duplicate transaction
				trTransactionResult.PreviousMessage = szPreviousMessage; // status code of original transaction if duplicate transaction
				trTransactionResult.CrossReference = szCrossReference;	// cross reference of transaction
				trTransactionResult.Amount = nAmount; // amount echoed back
				trTransactionResult.CurrencyCode = nCurrencyCode; // currency code echoed back
				trTransactionResult.OrderID = szOrderID; // order ID echoed back
				trTransactionResult.TransactionType = szTransactionType; // transaction type echoed back
				trTransactionResult.TransactionDateTime = szTransactionDateTime; // transaction date/time echoed back
				trTransactionResult.OrderDescription = szOrderDescription; // order description echoed back
				// the customer details that were actually
				// processed (might be different
				// from those passed to the payment form)
				trTransactionResult.CustomerName = szCustomerName;
				trTransactionResult.Address1 = szAddress1;
				trTransactionResult.Address2 = szAddress2;
				trTransactionResult.Address3 = szAddress3;
				trTransactionResult.Address4 = szAddress4;
				trTransactionResult.City = szCity;
				trTransactionResult.State = szState;
				trTransactionResult.PostCode = szPostCode;
				trTransactionResult.CountryCode = nCountryCode;
			}
		}
		catch (Any e)
		{
			boErrorOccurred = true;
			szOutputMessage.Value = e.message;
		}


		if (boErrorOccurred EQ true)
		{
			boReturnValue = false;
		}
		else
		{
			boReturnValue = true;
		}

		return (boReturnValue);
	}
	function AddStringToStringList(szExistingStringList, szStringToAdd)
	{
		szReturnString = "";
		szCommaString = "";

		if (Len(szStringToAdd) EQ 0)
		{
			szReturnString = szExistingStringList;
		}
		else
		{
			if (Len(szExistingStringList) NEQ 0)
			{
				szCommaString = ", ";
			}
			szReturnString = szExistingStringList & szCommaString & szStringToAdd;
		}

		return (szReturnString);
	}
    function GenerateStringToHash(szMerchantID,
    							  szPassword,
    							  szAmount,
								  szCurrencyCode,
								  szOrderID,
								  szTransactionType,
								  szTransactionDateTime,
								  szCallbackURL,
								  szOrderDescription,
								  szCustomerName,
								  szAddress1,
								  szAddress2,
								  szAddress3,
								  szAddress4,
								  szCity,
								  szState,
								  szPostCode,
								  szCountryCode,
								  szCV2Mandatory,
								  szAddress1Mandatory,
								  szCityMandatory,
								  szPostCodeMandatory,
								  szStateMandatory,
								  szCountryMandatory,
								  szResultDeliveryMethod,
								  szServerResultURL,
								  szPaymentFormDisplaysResult,
     		                      szPreSharedKey,
     		                      szHashMethod)
    {
		szReturnString = "";

		switch (szHashMethod)
		{
			case "MD5":
				boIncludePreSharedKeyInString = true;
				break;
			case "SHA1":
				boIncludePreSharedKeyInString = true;
				break;
			case "HMACMD5":
				boIncludePreSharedKeyInString = false;
				break;
			case "HMACSHA1":
				boIncludePreSharedKeyInString = false;
				break;
		}

		if (boIncludePreSharedKeyInString)
		{
			szReturnString = "PreSharedKey=" & szPreSharedKey & "&";
		}

		szReturnString = szReturnString & "MerchantID=" & szMerchantID & 
										  "&Password=" & szPassword & 
										  "&Amount=" & szAmount & 
										  "&CurrencyCode=" & szCurrencyCode & 
										  "&OrderID=" & szOrderID & 
										  "&TransactionType=" & szTransactionType & 
										  "&TransactionDateTime=" & szTransactionDateTime & 
										  "&CallbackURL=" & szCallbackURL & 
										  "&OrderDescription=" & szOrderDescription & 
										  "&CustomerName=" & szCustomerName & 
										  "&Address1=" & szAddress1 & 
										  "&Address2=" & szAddress2 & 
										  "&Address3=" & szAddress3 & 
										  "&Address4=" & szAddress4 & 
										  "&City=" & szCity & 
										  "&State=" & szState & 
										  "&PostCode=" & szPostCode & 
										  "&CountryCode=" & szCountryCode & 
										  "&CV2Mandatory=" & szCV2Mandatory & 
										  "&Address1Mandatory=" & szAddress1Mandatory & 
										  "&CityMandatory=" & szCityMandatory & 
										  "&PostCodeMandatory=" & szPostCodeMandatory & 
										  "&StateMandatory=" & szStateMandatory & 
										  "&CountryMandatory=" & szCountryMandatory & 
										  "&ResultDeliveryMethod=" & szResultDeliveryMethod & 
										  "&ServerResultURL=" & szServerResultURL & 
										  "&PaymentFormDisplaysResult=" & szPaymentFormDisplaysResult & 
										  "&ServerResultURLCookieVariables=" & 
										  "&ServerResultURLFormVariables=" & 
										  "&ServerResultURLQueryStringVariables=";

        return (szReturnString);
    }
    function GenerateStringToHash2(szMerchantID,
								   szPassword,
								   trTransactionResult,
     		                       szPreSharedKey,
     		                       szHashMethod)
    {
		szReturnString = "";

		switch (szHashMethod)
		{
			case "MD5":
				boIncludePreSharedKeyInString = true;
				break;
			case "SHA1":
				boIncludePreSharedKeyInString = true;
				break;
			case "HMACMD5":
				boIncludePreSharedKeyInString = false;
				break;
			case "HMACSHA1":
				boIncludePreSharedKeyInString = false;
				break;
		}

		if (boIncludePreSharedKeyInString)
		{
			szReturnString = "PreSharedKey=" & szPreSharedKey & "&";
		}

		szReturnString = szReturnString & "MerchantID=" & szMerchantID &
										  "&Password=" & szPassword &
										  "&StatusCode=" & trTransactionResult.StatusCode &
										  "&Message=" & trTransactionResult.Message &
										  "&PreviousStatusCode=" & trTransactionResult.PreviousStatusCode &
										  "&PreviousMessage=" & trTransactionResult.PreviousMessage &
										  "&CrossReference=" & trTransactionResult.CrossReference &
										  "&Amount=" & trTransactionResult.Amount &
										  "&CurrencyCode=" & trTransactionResult.CurrencyCode &
										  "&OrderID=" & trTransactionResult.OrderID &
										  "&TransactionType=" & trTransactionResult.TransactionType &
										  "&TransactionDateTime=" & trTransactionResult.TransactionDateTime &
										  "&OrderDescription=" & trTransactionResult.OrderDescription &
										  "&CustomerName=" & trTransactionResult.CustomerName &
										  "&Address1=" & trTransactionResult.Address1 &
										  "&Address2=" & trTransactionResult.Address2 &
										  "&Address3=" & trTransactionResult.Address3 &
										  "&Address4=" & trTransactionResult.Address4 &
										  "&City=" & trTransactionResult.City &
										  "&State=" & trTransactionResult.State &
										  "&PostCode=" & trTransactionResult.PostCode &
										  "&CountryCode=" & trTransactionResult.CountryCode;

        return (szReturnString);
    }
    function GenerateStringToHash3(szMerchantID,
								   szPassword,
								   szCrossReference,
								   szOrderID,
     		                       szPreSharedKey,
     		                       szHashMethod)
    {
		szReturnString = "";

		switch (szHashMethod)
		{
			case "MD5":
				boIncludePreSharedKeyInString = true;
				break;
			case "SHA1":
				boIncludePreSharedKeyInString = true;
				break;
			case "HMACMD5":
				boIncludePreSharedKeyInString = false;
				break;
			case "HMACSHA1":
				boIncludePreSharedKeyInString = false;
				break;
		}

		if (boIncludePreSharedKeyInString)
		{
			szReturnString = "PreSharedKey=" & szPreSharedKey & "&";
		}

		szReturnString = szReturnString & "MerchantID=" & szMerchantID &
										  "&Password=" & szPassword &
										  "&CrossReference=" & szCrossReference &
										  "&OrderID=" & szOrderID;

        return (szReturnString);
    }
	function ValidateTransactionResult_POST(szMerchantID, 
											szPassword, 
											szPreSharedKey, 
											szHashMethod,
											aPostVariables,
											trTransactionResult,
											szValidateErrorMessage)
	{
		boErrorOccurred = false;
		boReturnValue = false;
		szValidateErrorMessage.Value = "";

		szHashDigest = StructNew();
		szOutputMessage = StructNew();

		// read the transaction result variables from the post variable list
		if (GetTransactionResultFromPostVariables(aPostVariables, trTransactionResult, szHashDigest, szOutputMessage) NEQ true)
		{
			boErrorOccurred = true;
			szValidateErrorMessage.Value = szOutputMessage.Value;
		}
		else
		{
			// now need to validate the hash digest
			szStringToHash = GenerateStringToHash2(szMerchantID, szPassword, trTransactionResult, szPreSharedKey, szHashMethod);
			szCalculatedHashDigest = CalculateHashDigest(szStringToHash, szPreSharedKey, szHashMethod);
		
			// does the calculated hash match the one that was passed?
			if (UCase(szHashDigest.Value) NEQ UCase(szCalculatedHashDigest))
			{
				boErrorOccurred = true;
				szValidateErrorMessage.Value = "Hash digests don't match - possible variable tampering";
			}
			else
			{
				boErrorOccurred = false;
			}
		}

		if (boErrorOccurred EQ true)
		{
			boReturnValue = false;
		}
		else
		{
			boReturnValue = true;
		}

		return (boReturnValue);
	}
	function ValidateTransactionResult_SERVER(szMerchantID, 
											  szPassword, 
											  szPreSharedKey, 
											  szHashMethod,
											  aQueryStringVariables,
											  trTransactionResult,
											  szValidateErrorMessage)
	{
		boErrorOccurred = false;
		boReturnValue = false;
		szValidateErrorMessage.Value = "";

		szCrossReference = StructNew();
		szOrderID = StructNew();
		szHashDigest = StructNew();
		szOutputMessage = StructNew();

		// read the transaction reference variables from the query string variable list
		if (GetTransactionReferenceFromQueryString(aQueryStringVariables, szCrossReference, szOrderID, szHashDigest, szOutputMessage) NEQ true)
		{
			boErrorOccurred = true;
			szValidateErrorMessage.Value = szOutputMessage.Value;
		}
		else
		{
			// now need to validate the hash digest
			szStringToHash = GenerateStringToHash3(szMerchantID, szPassword, szCrossReference.Value, szOrderID.Value, szPreSharedKey, szHashMethod);
			szCalculatedHashDigest = CalculateHashDigest(szStringToHash, szPreSharedKey, szHashMethod);
		
			// does the calculated hash match the one that was passed?
			if (UCase(szHashDigest.Value) NEQ UCase(szCalculatedHashDigest))
			{
				boErrorOccurred = true;
				szValidateErrorMessage.Value = "Hash digests don't match - possible variable tampering";
			}
			else
			{
				// use the cross reference and/or the order ID to pull the
				// transaction results out of storage
				szOutputMessage = StructNew();

				if (GetTransactionResultFromStorage(szCrossReference,
													szOrderID,
													szOutputMessage) NEQ true)
				{
					szValidateErrorMessage.Value = szOutputMessage.Value;
					boErrorOccurred = true;
				}
				else
				{
					boErrorOccurred = false;
				}
			}
		}

		if (boErrorOccurred EQ true)
		{
			boReturnValue = false;
		}
		else
		{
			boReturnValue = true;
		}

		return (boReturnValue);
	}
	function ValidateTransactionResult_SERVER_PULL(szMerchantID, 
												   szPassword, 
												   szPreSharedKey, 
												   szHashMethod,
												   aQueryStringVariables,
												   szPaymentFormResultHandlerURL,
												   trTransactionResult,
												   szValidateErrorMessage)
	{
		boErrorOccurred = false;
		boReturnValue = false;
		szValidateErrorMessage.Value = "";

		szCrossReference = StructNew();
		szOrderID = StructNew();
		szHashDigest = StructNew();
		szOutputMessage = StructNew();

		// read the transaction reference variables from the query string variable list
		if (GetTransactionReferenceFromQueryString(aQueryStringVariables, szCrossReference, szOrderID, szHashDigest, szOutputMessage) NEQ true)
		{
			boErrorOccurred = true;
			szValidateErrorMessage.Value = szOutputMessage.Value;
		}
		else
		{
			// now need to validate the hash digest
			szStringToHash = GenerateStringToHash3(szMerchantID, szPassword, szCrossReference.Value, szOrderID.Value, szPreSharedKey, szHashMethod);
			szCalculatedHashDigest = CalculateHashDigest(szStringToHash, szPreSharedKey, szHashMethod);
		
			// does the calculated hash match the one that was passed?
			if (UCase(szHashDigest.Value) NEQ UCase(szCalculatedHashDigest))
			{
				boErrorOccurred = true;
				szValidateErrorMessage.Value = "Hash digests don't match - possible variable tampering";
			}
			else
			{
				// use the cross reference and/or the order ID to pull the
				// transaction results out of storage
				szOutputMessage = StructNew();

				if (GetTransactionResultFromPaymentFormHandler(szPaymentFormResultHandlerURL,
															   szMerchantID, 
															   szPassword,
															   szCrossReference.Value,
															   trTransactionResult,
															   szOutputMessage) NEQ true)
				{
					szValidateErrorMessage.Value = "Error querying transaction result [" & szCrossReference.Value & "] from [" & szPaymentFormResultHandlerURL & "]: " & szOutputMessage.Value;
					boErrorOccurred = true;
				}
				else
				{
					boErrorOccurred = false;
				}
			}
		}

		if (boErrorOccurred EQ true)
		{
			boReturnValue = false;
		}
		else
		{
			boReturnValue = true;
		}

		return (boReturnValue);
	}
	function SplitByToken(szStringToSplit, szToken)
	{
		boFinished = false;
		nIndex = 0;

		aOutput = ArrayNew(1);	
		szWorkingString = szStringToSplit;

		while (boFinished NEQ true)
		{
			nIndex = Find(szToken, szWorkingString);

			if (nIndex EQ 0)
			{
				boFinished = true;
				if (szWorkingString NEQ "")
				{
					ArrayAppend(aOutput, szWorkingString);
				}
			}
			else
			{
				szVariableString = Mid(szWorkingString, 1, nIndex - 1);
				if (szVariableString NEQ "")
				{
					ArrayAppend(aOutput, szVariableString);
				}
				szWorkingString = Mid(szWorkingString, nIndex + 1, Len(szWorkingString) - nIndex);
			}
		}

		return (aOutput);
	}
	function ParseNameValueStringIntoArray(szNameValueString, boURLDecodeValues)
	{
		aParsedVariables = StructNew();

		aStrings = SplitByToken(szNameValueString, "&");

		for (nCount = 1; nCount LTE ArrayLen(aStrings); nCount = nCount +1)
		{
			aVariables = SplitByToken(aStrings[nCount], "=");

			szVariableName = aVariables[1];

			if (ArrayLen(aVariables) EQ 1)
			{
				szVariableValue = "";				
			}
			else
			{
				if (boURLDecodeValues)
				{
					szVariableValue = URLDecode(aVariables[2]);				
				}
				else
				{
					szVariableValue = aVariables[2];
				}
			}
			SetVariable("aParsedVariables." & szVariableName, szVariableValue);
		}

		return (aParsedVariables);
	}
	function FindVariable(aVariableArray, szName)
	{
		szValue = "";
		nCount = 0;
		boFound = false;		

		while (boFound NEQ true	AND
			   nCount LTE ArrayLen(aVariableArray))
		{
			try
			{
				if (aVariableArray[nCount].Name EQ szName)
				{
					boFound = true;
					szValue = aVariableArray[nCount].Value;
				}
			}
			catch (Any e)
			{
			}
			nCount = nCount + 1;
		}

		return (szValue);
	}
	function GetTransactionResultFromPaymentFormHandler(szPaymentFormResultHandlerURL,
														szMerchantID,
														szPassword,
														szCrossReference,
														trTransactionResult,
														szOutputMessage)
	{
		boErrorOccurred = false;
		boReturnValue = false;
		szOutputMessage.Value = "";

		try
		{
			// post the cross reference to the
			// payment form to query its status
		    aPostVariables = ArrayNew(1);
		    sPostVariable = StructNew();
		    sPostVariable.Name = "MerchantID";
		    sPostVariable.Value = URLDecode(szMerchantID);
		    ArrayAppend(aPostVariables, sPostVariable);
		    sPostVariable = StructNew();
		    sPostVariable.Name = "Password";
		    sPostVariable.Value = URLDecode(szPassword);
		    ArrayAppend(aPostVariables, sPostVariable);
		    sPostVariable = StructNew();
		    sPostVariable.Name = "CrossReference";
		    sPostVariable.Value = URLDecode(szCrossReference);
		    ArrayAppend(aPostVariables, sPostVariable);

			szResponse = HTTPPost(szPaymentFormResultHandlerURL, aPostVariables);

			if (szResponse EQ "")
			{
				boErrorOccurred = true;
				szOutputMessage.Value = "Received empty response from payment form hander";
			}
			else
			{
				try
				{
					// parse the response into an array
					aParsedPostVariables = ParseNameValueStringIntoArray(szResponse, true);

					if (isDefined("aParsedPostVariables.StatusCode") EQ false OR
						aParsedPostVariables.StatusCode NEQ 0)
					{
						boErrorOccurred = true;

						// the message field is expected if the status code is non-zero
						if (isDefined("aParsedPostVariables.Message") NEQ true)
						{
							szOutputMessage.Value = "Received invalid response from payment form hander [" & szResponse & "]";
						}
						else
						{
							szOutputMessage.Value = aParsedPostVariables.Message;
						}
					}
					else
					{
						// status code is 0, so	get the transaction result
						if (isDefined("aParsedPostVariables.TransactionResult") NEQ true)
						{
							boErrorOccurred = true;
							szOutputMessage.Value = "No transaction result in response from payment form hander [" & szResponse & "]";
						}
						else
						{
							// parse the URL decoded transaction result field into a name value array
							aTransactionResultArray = ParseNameValueStringIntoArray(URLDecode(aParsedPostVariables.TransactionResult), false);

							// parse this array into a transaction result object
							szHashDigest = StructNew();
							szErrorMessage = StructNew();

							if (GetTransactionResultFromPostVariables(aTransactionResultArray, trTransactionResult, szHashDigest, szErrorMessage) NEQ true)
							{
								boErrorOccurred = true;
								szOutputMessage.Value = "Error [" & szErrorMessage.Value & "] parsing transaction result [" & URLDecode(aParsedPostVariables.TransactionResult) & "] in response from payment form hander [" & szResponse & "]";
							}
							else
							{
								boErrorOccurred = false;
							}
						}
					}
				}
				catch (Any e)
				{
					boErrorOccurred = true;
					szOutputMessage.Value = "Exception [" & e.message & "] when processing response from payment form handler [" & szResponse & "]";
				}
			}
		}
		catch (Any e)
		{
			boErrorOccurred = true;
			szOutputMessage.Value = e.message;
		}

		if (boErrorOccurred EQ true)
		{
			boReturnValue = false;
		}
		else
		{
			boReturnValue = true;
		}

		return (boReturnValue);
	}

    // These functions that are run to deal with storing and retrieving the
	// transaction results. They will be specific to the merchant environment, so cannot
	// be generalised. The developer MUST implement these functions

	// This function needs to be able to retrieve the saved transaction resultt
	// so that the result can be displayed to the customer
	function GetTransactionResultFromStorage(szCrossReference,
											 szOrderID,
											 trTransactionResult,
											 szOutputMessage)
	{
		boErrorOccurred = true;
		boReturnValue = false;
		szOutputMessage.Value = "Environment specific function GetTransactionResultFromStorage() needs to be implemented by merchant developer";

		if (boErrorOccurred EQ true)
		{
			boReturnValue = false;
		}
		else
		{
			boReturnValue = true;
		}

		return (boReturnValue);
	}

	// You should put your code that does any post transaction tasks
	// (e.g. updates the order object, sends the customer an email etc) in this function
    function ReportTransactionResults(trTransactionResult, szOutputMessage)
    {
		boErrorOccurred = true;
		boReturnValue = false;
		szOutputMessage.Value = "Environment specific function ReportTransactionResults() needs to be implemented by merchant developer";

		try
		{
			switch (trTransactionResult.StatusCode)
			{
				// transaction authorised
				case 0:
					break;
				// card referred (treat as decline)
				case 4:
					break;
				// transaction declined
				case 5:
					break;
				// duplicate transaction
				case 20:
					// need to look at the previous status code to see if the
					// transaction was successful
					if (trTransactionResult.PreviousStatusCode EQ 0)
					{
						// transaction authorised
					}
					else
					{
						// transaction not authorised
					}
					break;
				// error occurred
				case 30:
					break;
				default:
					break;
			}

			// put code to update/store the order with the transaction result
		}
		catch (Any e)
		{
			boErrorOccurred = true;
			szOutputMessage.Value = e.message;
		}

		if (boErrorOccurred EQ true)
		{
			boReturnValue = false;
		}
		else
		{
			boReturnValue = true;
		}

		return (boReturnValue);
    }
</cfscript>