<cffunction name="IncludeTemplate">
	<cfargument name="templateName" type="string" required="true">

	<cfinclude template="#templateName#">
</cffunction>
<cffunction name="Abort">
	<cfargument name="output" type="string" required="false">
	<cfif isDefined("output") EQ true>
		<cfoutput>#output#</cfoutput>
	</cfif>
	<cfabort>
</cffunction>
<cffunction name="Dump">
	<cfargument name="var" type="any" required="true">
	<cfdump var="#var#">
</cffunction>
<cffunction name="HTTPPost" returnType="string">
	<cfargument name="url" type="string" required="true">
	<cfargument name="postVariables" type="array" required="true">
    
    <cfset returnString="">

    <cfif ArrayLen(postVariables) GT 0>
    	<cfhttp url="#url#" method="post">
            <cfloop index="nLoopIndex" from="1" to="#ArrayLen(postVariables)#">
                <cfhttpparam type="formfield" name="#postVariables[nLoopIndex].Name#" value="#postVariables[nLoopIndex].Value#">
            </cfloop>
        </cfhttp>

        <cfset returnString=cfhttp.fileContent>
    </cfif>
    <cfreturn returnString>
</cffunction>
<cffunction name="Throw">
	<cfargument name="message" type="string" required="true">

	<cfthrow type="any" message="#message#">
</cffunction>

<cfscript>
	function CalculateHashDigest(szInputString, szPreSharedKey, szHashMethod)
    {
		szCalculatedHashDigest = "";

		try
		{
			switch (szHashMethod)
			{
				case "MD5":
	        		szCalculatedHashDigest = Hash(szInputString, "MD5");
					break;
				case "SHA1":
	        		szCalculatedHashDigest = Hash(szInputString, "SHA1");
					break;
				case "HMACMD5":
				case "HMACSHA1":
					Throw("Hash method [" & szHashMethod & "] not supported with this version of ColdFusion [" & SERVER.ColdFusion.ProductVersion & "]");	
					break;
			}
		}
		catch (Any e)
		{
			Throw("Hash method [" & szHashMethod & "] not supported with this version of ColdFusion [" & SERVER.ColdFusion.ProductVersion & "]");	
		}

        return (szCalculatedHashDigest);
    }
	function GetTimeZoneOffset()
	{
		szReturnString = "";
		szPlusMinusSign = "+";
		szHourOffset = "00";
		szMinuteOffset = "00";

		iInfo = GetTimeZoneInfo();
		nTotalOffset = iInfo.utcTotalOffset;

		if (nTotalOffset LT 0)
		{
			szPlusMinusSign = "-";
			nTotalOffset = nTotalOffset * -1;
		}

		nMinuteOffset = nTotalOffset Mod 60;
		nHourOffset = (nTotalOffset - nMinuteOffset) / 60;

		if (nHourOffset LT 10)
		{
			szHourOffset = "0" & nHourOffset;
		}
		else
		{
			szHourOffset = nHourOffset;
		}

		if (nMinuteOffset LT 10)
		{
			szMinuteOffset = "0" & nMinuteOffset;
		}
		else
		{
			szMinuteOffset = nMinuteOffset;
		}

		szReturnString = szPlusMinusSign & szHourOffset & ":" & szMinuteOffset;

		return (szReturnString);
	}
	function GetSiteSecureBaseURL()
	{
		szReturnString = "";
		szPortString = "";
		szProtocolString = "";

		if (CGI.HTTPS EQ "on")
		{
			szProtocolString = "https://";
			if (CGI.SERVER_PORT NEQ 443)
			{
				szPortString = ":" & CGI.SERVER_PORT;
			}
		}
		else
		{
			szProtocolString = "http://";			
			if (CGI.SERVER_PORT NEQ 80)
			{
				szPortString = ":" & CGI.SERVER_PORT;
			}
		}

		szReturnString = szProtocolString & CGI.SERVER_NAME & szPortString & CGI.SCRIPT_NAME;
		
		boFinished = false;
		nLoopIndex = Len(szReturnString);

		while (boFinished EQ false AND
			   nLoopIndex GTE 1)
		{
			if (Mid(szReturnString, nLoopIndex, 1) EQ "/")
			{
				boFinished = true;
				szReturnString = Mid(szReturnString, 1, nLoopIndex);
			}
			nLoopIndex = nLoopIndex - 1;
		}

		return (szReturnString);
	}

	
	function AddStringToStringList(szExistingStringList, szStringToAdd)
	{
		szReturnString = "";
		szCommaString = "";

		if (Len(szStringToAdd) EQ 0)
		{
			szReturnString = szExistingStringList;
		}
		else
		{
			if (Len(szExistingStringList) NEQ 0)
			{
				szCommaString = ", ";
			}
			szReturnString = szExistingStringList & szCommaString & szStringToAdd;
		}

		return (szReturnString);
	}
	function IncludePreSharedKey(szHashMethod)
	{
		boIncludePreSharedKeyInString = true;
		switch (szHashMethod)
		{
			case "HMACMD5":
			case "HMACSHA1":
				boIncludePreSharedKeyInString = false;
				break;
		}
		return boIncludePreSharedKeyInString;
	}
    
	function SplitByToken(szStringToSplit, szToken)
	{
		boFinished = false;
		nIndex = 0;

		aOutput = ArrayNew(1);	
		szWorkingString = szStringToSplit;

		while (boFinished NEQ true)
		{
			nIndex = Find(szToken, szWorkingString);

			if (nIndex EQ 0)
			{
				boFinished = true;
				if (szWorkingString NEQ "")
				{
					ArrayAppend(aOutput, szWorkingString);
				}
			}
			else
			{
				szVariableString = Mid(szWorkingString, 1, nIndex - 1);
				if (szVariableString NEQ "")
				{
					ArrayAppend(aOutput, szVariableString);
				}
				szWorkingString = Mid(szWorkingString, nIndex + 1, Len(szWorkingString) - nIndex);
			}
		}

		return (aOutput);
	}
	function ParseNameValueStringIntoArray(szNameValueString, boURLDecodeValues)
	{
		aParsedVariables = StructNew();

		aStrings = SplitByToken(szNameValueString, "&");

		for (nCount = 1; nCount LTE ArrayLen(aStrings); nCount = nCount +1)
		{
			aVariables = SplitByToken(aStrings[nCount], "=");

			szVariableName = aVariables[1];

			if (ArrayLen(aVariables) EQ 1)
			{
				szVariableValue = "";				
			}
			else
			{
				if (boURLDecodeValues)
				{
					szVariableValue = URLDecode(aVariables[2]);				
				}
				else
				{
					szVariableValue = aVariables[2];
				}
			}
			SetVariable("aParsedVariables." & szVariableName, szVariableValue);
		}

		return (aParsedVariables);
	}
	function FindVariable(aVariableArray, szName)
	{
		szValue = "";
		nCount = 0;
		boFound = false;		

		while (boFound NEQ true	AND
			   nCount LTE ArrayLen(aVariableArray))
		{
			try
			{
				if (aVariableArray[nCount].Name EQ szName)
				{
					boFound = true;
					szValue = aVariableArray[nCount].Value;
				}
			}
			catch (Any e)
			{
			}
			nCount = nCount + 1;
		}

		return (szValue);
	}
	function GetJavaFriendlyCurrencyShort(iclISOCurrencyList, CurrencyShort)
	{
		szReturnString = "";
		boFound = false;
		LoopIndex = 0;

		while (boFound EQ false AND
			   LoopIndex LT iclISOCurrencyList.Count)
		{
			if (iclISOCurrencyList.getAt(JavaCast("int", LoopIndex)).CurrencyShort EQ CurrencyShort)
			{
				szReturnString = iclISOCurrencyList.getAt(JavaCast("int", LoopIndex)).CurrencyShort;
				boFound = true;
			}
			LoopIndex=LoopIndex+1;
		}				
		return (szReturnString);
	}
	function GetJavaFriendlyCountryShort(iclISOCountryList, CountryShort)
	{
		szReturnString = "";
		boFound = false;
		LoopIndex = 0;

		while (boFound EQ false AND
			   LoopIndex LT iclISOCountryList.Count)
		{
			if (iclISOCountryList.getAt(JavaCast("int", LoopIndex)).CountryShort EQ CountryShort)
			{
				szReturnString = iclISOCountryList.getAt(JavaCast("int", LoopIndex)).CountryShort;
				boFound = true;
			}
			LoopIndex=LoopIndex+1;
		}				
		return (szReturnString);
	}
	function ListItemArrayToString(aListItemList)
	{
		szReturnString = "";

		for (LoopIndex = 1; LoopIndex LTE ArrayLen(aListItemList); LoopIndex=LoopIndex+1)
		{
			szReturnString = szReturnString & "<option";

			if (aListItemList[LoopIndex].name NEQ "")
			{
				szReturnString = szReturnString & " value=""" & aListItemList[LoopIndex].value & """";
			}

			if (aListItemList[LoopIndex].selected EQ true)
			{
				szReturnString = szReturnString & " selected=""selected""";	
			}
			szReturnString = szReturnString & ">" & aListItemList[LoopIndex].name & "</option>";
		}

		return (szReturnString);
	}
	function CreateExpiryDateMonthListItemList(ExpiryDateMonth)
	{
		aExpiryDateMonthList = ArrayNew(1);

		for (LoopIndex = 1; LoopIndex LTE 12; LoopIndex=LoopIndex+1)
		{
			DisplayMonth = LoopIndex;
			if (LoopIndex LT 10)
			{
				DisplayMonth = "0" & LoopIndex;
			}
			
			sListItem = StructNew();
			sListItem.name = DisplayMonth;
			sListItem.value = DisplayMonth;

			if (ExpiryDateMonth NEQ "" AND
			    ExpiryDateMonth EQ LoopIndex)
			{
				sListItem.selected = true;
			}
			else
			{
				sListItem.selected = false;
			}
			ArrayAppend(aExpiryDateMonthList, sListItem);
		}

		return (aExpiryDateMonthList);
	}
	function CreateExpiryDateYearListItemList(ExpiryDateYear)
	{
		ThisYear = DatePart("yyyy", Now());
		ThisYearPlusTen = ThisYear + 10;
	
		aExpiryDateYearList = ArrayNew(1);

		for (LoopIndex = ThisYear; LoopIndex LTE ThisYearPlusTen; LoopIndex=LoopIndex+1)
		{
			ShortYear = MID(LoopIndex, Len(LoopIndex)-1, 2);

			sListItem = StructNew();
			sListItem.name = LoopIndex;
			sListItem.value = ShortYear;

			if (ExpiryDateYear NEQ "" AND
			    ExpiryDateYear EQ ShortYear)
			{
				sListItem.selected = true;
			}
			else
			{
				sListItem.selected = false;
			}
			ArrayAppend(aExpiryDateYearList, sListItem);
		}

		return (aExpiryDateYearList);
	}
	function CreateStartDateMonthListItemList(StartDateMonth)
	{
		aStartDateMonthList = ArrayNew(1);

		for (LoopIndex = 1; LoopIndex LTE 12; LoopIndex=LoopIndex+1)
		{
			DisplayMonth = LoopIndex;
			if (LoopIndex LT 10)
			{
				DisplayMonth = "0" & LoopIndex;
			}
			
			sListItem = StructNew();
			sListItem.name = DisplayMonth;
			sListItem.value = DisplayMonth;

			if (StartDateMonth NEQ "" AND
			    StartDateMonth EQ LoopIndex)
			{
				sListItem.selected = true;
			}
			else
			{
				sListItem.selected = false;
			}
			ArrayAppend(aStartDateMonthList, sListItem);
		}

		return (aStartDateMonthList);
	}
   	function CreateStartDateYearListItemList(StartDateYear)
	{
		ThisYear = DatePart("yyyy", Now());

		aStartDateYearList = ArrayNew(1);

		for (LoopIndex = ThisYear-10; LoopIndex LTE ThisYear; LoopIndex=LoopIndex+1)
	   	{
			ShortYear = MID(LoopIndex, Len(LoopIndex)-1, 2);

			sListItem = StructNew();
			sListItem.name = LoopIndex;
			sListItem.value = ShortYear;

	   		if (StartDateYear NEQ "" AND
	   	    	StartDateYear EQ ShortYear)
	   		{
				sListItem.selected = true;
			}
			else
			{
				sListItem.selected = false;
			}
			ArrayAppend(aStartDateYearList, sListItem);
		}
		
		return (aStartDateYearList);		
	}
	function CreateISOCountryListItemList()
	{
		aISOCountryList = ArrayNew(1);

		sListItem = StructNew();
		sListItem.name = "United Kingdom";
		sListItem.value = "826";
		sListItem.selected = true;
		ArrayAppend(aISOCountryList, sListItem);
		
		// TODO ISOCOUNTRY LOOP
		
		return aISOCountryList;
	}
	function SetVariableInArray(aArray, szVariableName, szVariableValue)
	{
		sVariableOutput = GetVariableInArray(aArray, szVariableName);

		if (sVariableOutput.index NEQ -1)
		{
			aArray[sVariableOutput.index].value = szVariableValue;
		}
		else
		{
			sVariable = StructNew();
			sVariable.name = szVariableName;
			sVariable.value = szVariableValue;

			ArrayAppend(aArray, sVariable);
		}

		return(aArray);
	}
	function GetVariableInArray(aArray, szVariableName)
	{
		sReturnValue = StructNew();
		sReturnValue.value = "";
		sReturnValue.index = -1;
		LoopIndex = 1;
		boFound = false;

		while (boFound EQ false AND 
			   LoopIndex LTE ArrayLen(aArray))
		{
			if (aArray[LoopIndex].name EQ szVariableName)
			{
				boFound = true;
				sReturnValue.index = LoopIndex;
				sReturnValue.value = aArray[LoopIndex].value;
			}
		
			LoopIndex=LoopIndex+1;
		}

		return (sReturnValue);
	}
    function GenerateStringToHash(szAmount,
								  szCurrencyCode,
								  szOrderID,
								  szTransactionType,
								  szTransactionDateTime,
								  szCallbackURL,
								  szOrderDescription,
								  szCustomerName,
								  szAddress1,
								  szAddress2,
								  szAddress3,
								  szAddress4,
								  szCity,
								  szState,
								  szPostCode,
								  szCountryCode,
								  szSecretKey)
    {
		szReturnString = "";

		szReturnString = szReturnString & "&Amount=" & szAmount & 
										  "&CurrencyCode=" & szCurrencyCode & 
										  "&OrderID=" & szOrderID & 
										  "&TransactionType=" & szTransactionType & 
										  "&TransactionDateTime=" & szTransactionDateTime & 
										  "&CallbackURL=" & szCallbackURL & 
										  "&OrderDescription=" & szOrderDescription & 
										  "&CustomerName=" & szCustomerName & 
										  "&Address1=" & szAddress1 & 
										  "&Address2=" & szAddress2 & 
										  "&Address3=" & szAddress3 & 
										  "&Address4=" & szAddress4 & 
										  "&City=" & szCity & 
										  "&State=" & szState & 
										  "&PostCode=" & szPostCode & 
										  "&CountryCode=" & szCountryCode & 
										  "&SecretKey=" & szSecretKey;

        return (szReturnString);
    }
	function CheckIntegrityOfIncomingVariables(ivsIncomingVariableSource, aVariables, szHashMethod, szSecretKeyOrPreSharedKey, szMerchantID, szPassword)
	{
		boReturnValue = false;
		szHash = "";
		szStringToHash = "";
		szCalculatedHash = "";

		switch (ivsIncomingVariableSource)
		{
			case "SHOPPING_CART_CHECKOUT":	
				szHash = aVariables.ShoppingCartHashDigest;		
                szStringToHash = GenerateStringToHash(	aVariables.Amount,
								                		aVariables.CurrencyCode,
								                		aVariables.OrderID,
								                		aVariables.TransactionType,
								                		aVariables.TransactionDateTime,
								                		aVariables.CallbackURL,
								                		aVariables.OrderDescription,
								                		aVariables.CustomerName,
								                		aVariables.Address1,
								                		aVariables.Address2,
								                		aVariables.Address3,
								                		aVariables.Address4,
								                		aVariables.City,
								                		aVariables.State,
								                		aVariables.PostCode,
								                		aVariables.CountryCode,
								                		szSecretKeyOrPreSharedKey);
				break;
			case "THREE_D_SECURE_AUTHENTICATION_REQUIRED" :
				// read in the transaction result variables
				trTransactionResult = StructNew();
				szHashDigest = StructNew();
				szOutputMessage = StructNew();

				szHash = aVariables.HashDigest;		
                Get3DSecureAuthenticationRequiredFromPostVariables(	aVariables, 
                													trTransactionResult, 
                													szHashDigest, 
                													szOutputMessage);
                
                szStringToHash = GenerateStringToHash3DSecureAuthenticationRequired(szMerchantID,
                																	szPassword, 
                																	trTransactionResult, 
                																	szSecretKeyOrPreSharedKey, 
                																	szHashMethod);
                break;
			case "RESULTS" :
				// read in the transaction result variables
				trTransactionResult = StructNew();
				szHashDigest = StructNew();
				szOutputMessage = StructNew();

				szHash = aVariables.HashDigest;		
                GetTransactionCompleteResultFromPostVariables(	aVariables, 
                												trTransactionResult, 
                												szHashDigest, 
                												szOutputMessage);
                												
                szStringToHash = GenerateStringToHashPaymentComplete(	szMerchantID, 
                														szPassword, 
                														trTransactionResult, 
                														szSecretKeyOrPreSharedKey, 
                														szHashMethod);
                break;
		}
		
		szCalculatedHash = CalculateHashDigest(szStringToHash, szSecretKeyOrPreSharedKey, szHashMethod);
		
		if (szCalculatedHash EQ szHash)
		{
			boReturnValue = true;
		}
		return (boReturnValue);
	}

	function GenerateStringToHashInitial(szMerchantID,
										szPassword,
										szAmount,
										nCurrencyCode,
										szOrderID,
										szTransactionType,
										szTransactionDateTime,
										szCallbackURL,
										szOrderDescription,
										szHashMethod,
										szPreSharedKey) 
		{

		szReturnString = "";

		if (IncludePreSharedKey(szHashMethod))
		{
			szReturnString = "PreSharedKey=" & szPreSharedKey & "&";
		}

		szReturnString = szReturnString & 
          	"MerchantID=" & szMerchantID &
          	"&Password=" & szPassword &
          	"&Amount=" & szAmount &
          	"&CurrencyCode=" & nCurrencyCode &
          	"&OrderID=" & szOrderID & 
          	"&TransactionType=" & szTransactionType &
          	"&TransactionDateTime=" & szTransactionDateTime &
          	"&CallbackURL=" & szCallbackURL &
          	"&OrderDescription=" & szOrderDescription;

		return szReturnString;
    }

	function GenerateStringToHash3DSecureAuthenticationRequired(szMerchantID,
   																szPassword,
   																trTransactionResult,
   																szPreSharedKey,
   																szHashMethod)
	{

		szReturnString = "";

		if (IncludePreSharedKey(szHashMethod))
		{
			szReturnString = "PreSharedKey=" & szPreSharedKey & "&";
		}
		
		szReturnString = szReturnString & 
			"MerchantID=" & szMerchantID &
			"&Password=" & szPassword &
			"&StatusCode=" & trTransactionResult.StatusCode &
			"&Message=" & trTransactionResult.Message &
			"&CrossReference=" & trTransactionResult.CrossReference &
			"&OrderID=" & trTransactionResult.OrderID &
			"&TransactionDateTime=" & trTransactionResult.TransactionDateTime &
			"&ACSURL=" & trTransactionResult.ACSUrl &
			"&PaREQ=" & trTransactionResult.PaREQ;
		
       	return szReturnString;
    }

    function GenerateStringToHash3DSecurePostAuthentication(szMerchantID,
    														szPassword,
    														szCrossReference,
    														szTransactionDateTime,
    														szCallbackURL,
    														szPaRES,
    														szPreSharedKey,
    														szHashMethod)
	{
    																	    																	
        szReturnString = "";

		if (IncludePreSharedKey(szHashMethod))
		{
			szReturnString = "PreSharedKey=" & szPreSharedKey & "&";
		}
		
		szReturnString = szReturnString & 
			"MerchantID=" & szMerchantID &
			"&Password=" & szPassword &
			"&CrossReference=" & szCrossReference &
			"&TransactionDateTime=" & szTransactionDateTime &
			"&CallbackURL=" & szCallbackURL &
			"&PaRES=" & szPaRES;
		
		return szReturnString;
    }

    function GenerateStringToHashPaymentComplete(szMerchantID,
  												 szPassword,
    											 trTransactionResult,
    											 szPreSharedKey,
    											 szHashMethod)
	{
    							
		szReturnString = "";

		if (IncludePreSharedKey(szHashMethod))
		{
			szReturnString = "PreSharedKey=" & szPreSharedKey & "&";
		}
		
		szReturnString = szReturnString & 
			"MerchantID=" & szMerchantID &
			"&Password=" & szPassword &
			"&StatusCode=" & trTransactionResult.StatusCode &
			"&Message=" & trTransactionResult.Message &
			"&PreviousStatusCode=" & trTransactionResult.PreviousStatusCode &
			"&PreviousMessage=" & trTransactionResult.PreviousMessage &
			"&CrossReference=" & trTransactionResult.CrossReference &
			"&AddressNumericCheckResult=" & trTransactionResult.AddressNumericCheckResult &
			"&PostCodeCheckResult=" & trTransactionResult.PostCodeCheckResult &
			"&CV2CheckResult=" & trTransactionResult.CV2CheckResult &
			"&ThreeDSecureAuthenticationCheckResult=" & trTransactionResult.ThreeDSecureAuthenticationCheckResult &
			"&CardType=" & trTransactionResult.CardType &
			"&CardClass=" & trTransactionResult.CardClass &
			"&CardIssuer=" & trTransactionResult.CardIssuer &
			"&CardIssuerCountryCode=" & trTransactionResult.CardIssuerCountryCode &
			"&Amount=" & trTransactionResult.Amount &
			"&CurrencyCode=" & trTransactionResult.CurrencyCode &
			"&OrderID=" & trTransactionResult.OrderID &
			"&TransactionType=" & trTransactionResult.TransactionType &
			"&TransactionDateTime=" & trTransactionResult.TransactionDateTime &
			"&OrderDescription=" & trTransactionResult.OrderDescription &
			"&Address1=" & trTransactionResult.Address1 &
			"&Address2=" & trTransactionResult.Address2 &
			"&Address3=" & trTransactionResult.Address3 &
			"&Address4=" & trTransactionResult.Address4 &
			"&City=" & trTransactionResult.City &
			"&State=" & trTransactionResult.State &
			"&PostCode=" & trTransactionResult.PostCode &
			"&CountryCode=" & trTransactionResult.CountryCode &
			"&EmailAddress=" & trTransactionResult.EmailAddress &
			"&PhoneNumber=" & trTransactionResult.PhoneNumber;
		
		return szReturnString;
    }
    function Get3DSecureAuthenticationRequiredFromPostVariables(aFormVariables,
    															trTransactionResult,
    															szHashDigest,
    															szOutputMessage)
	{
	
		boErrorOccurred = false;
		boReturnValue = false;
		szOutputMessage.Value = "";
		szHashDigest.Value = "";

		try
		{
			// hash digest
			if (isDefined("aFormVariables.HashDigest") EQ true)
			{
				szHashDigest.Value = aFormVariables.HashDigest;
			}

			// transaction status code
			if (isDefined("aFormVariables.StatusCode") NEQ true)
			{
				szOutputMessage.Value = AddStringToStringList(szOutputMessage.Value, "Expected variable [StatusCode] not received");
				boErrorOccurred = true;
			}
			else
			{
				if (aFormVariables.StatusCode EQ "")
				{
					trTransactionResult.StatusCode = "";
				}
				else
				{
					trTransactionResult.StatusCode = aFormVariables.StatusCode;
				}
			}
			// transaction message
            if (isDefined("Message") NEQ true) {
                szOutputMessage.Value = AddStringToStringList(szOutputMessage.Value, "Expected variable [Message] not received");
                boErrorOccurred = true;
            } else {
                trTransactionResult.Message = aFormVariables.Message;
            }
            // cross reference of transaction
            if (isDefined("CrossReference") NEQ true) {
                szOutputMessage.Value = AddStringToStringList(szOutputMessage.Value, "Expected variable [CrossReference] not received");
                boErrorOccurred = true;
            } else {
                trTransactionResult.CrossReference = aFormVariables.CrossReference;
            }
            // order ID (same as value passed into payment form - echoed back out by payment form)
            if (isDefined("OrderID") NEQ true) {
                szOutputMessage.Value = AddStringToStringList(szOutputMessage.Value, "Expected variable [OrderID] not received");
                boErrorOccurred = true;
            } else {
                trTransactionResult.OrderID = aFormVariables.OrderID;
            }
            // transaction date/time (same as value passed into payment form - echoed back out by payment form)
            if (isDefined("TransactionDateTime") NEQ true) {
                szOutputMessage.Value = AddStringToStringList(szOutputMessage.Value, "Expected variable [TransactionDateTime] not received");
                boErrorOccurred = true;
            } else {
                trTransactionResult.TransactionDateTime = aFormVariables.TransactionDateTime;
            }
            // order description (same as value passed into payment form - echoed back out by payment form)
            if (isDefined("ACSURL") NEQ true) {
                szOutputMessage.Value = AddStringToStringList(szOutputMessage.Value, "Expected variable [ACSURL] not received");
                boErrorOccurred = true;
            } else {
                trTransactionResult.ACSURL = aFormVariables.ACSURL;
            }
            // address1 (not necessarily the same as value passed into payment form - as the customer can change it on the form)
            if (isDefined("PaREQ") NEQ true) {
                szOutputMessage.Value = AddStringToStringList(szOutputMessage.Value, "Expected variable [PaREQ] not received");
                boErrorOccurred = true;
            } else {
                trTransactionResult.PaREQ = aFormVariables.PaREQ;
            }

            if (boErrorOccurred) {
                trTransactionResult = null;
            }
        } catch (Any e) {
            boErrorOccurred = true;
            szOutputMessage.Value = e.getMessage();
        }

        if (boErrorOccurred EQ true)
		{
			boReturnValue = false;
		}
		else
		{
			boReturnValue = true;
		}

		return (boReturnValue);
    }

    function GetTransactionCompleteResultFromPostVariables(aFormVariables,
														   trTransactionResult,
														   szHashDigest,
														   szOutputMessage)
	{
        boErrorOccurred = false;
		boReturnValue = false;
		szOutputMessage.Value = "";
		szHashDigest.Value = "";

		try
		{
			// hash digest
			if (isDefined("aFormVariables.HashDigest") EQ true)
			{
				szHashDigest.Value = aFormVariables.HashDigest;
			}

            // transaction status code
            if (isDefined("aFormVariables.StatusCode") NEQ true) {
                szOutputMessage.Value = AddStringToStringList(szOutputMessage.Value, "Expected variable [StatusCode] not received");
                boErrorOccurred = true;
            } else {
				if (aFormVariables.StatusCode EQ "")
				{
					trTransactionResult.StatusCode = "";
				}
				else
				{
					trTransactionResult.StatusCode = aFormVariables.StatusCode;
				}
            }
            // transaction message
            if (isDefined("aFormVariables.Message") NEQ true) {
                szOutputMessage.Value = AddStringToStringList(szOutputMessage.Value, "Expected variable [Message] not received");
                boErrorOccurred = true;
            } else {
                trTransactionResult.Message = aFormVariables.Message;
            }
            // status code of original transaction if this transaction was deemed a duplicate
            if (isDefined("aFormVariables.PreviousStatusCode") NEQ true) {
                szOutputMessage.Value = AddStringToStringList(szOutputMessage.Value, "Expected variable [PreviousStatusCode] not received");
                boErrorOccurred = true;
            } else {
				if (aFormVariables.PreviousStatusCode EQ "")
				{
					trTransactionResult.PreviousStatusCode = "";
				}
				else
				{
					trTransactionResult.PreviousStatusCode = aFormVariables.PreviousStatusCode;
				}
            }
            // status code of original transaction if this transaction was deemed a duplicate
            if (isDefined("aFormVariables.PreviousMessage") NEQ true) {
                szOutputMessage.Value = AddStringToStringList(szOutputMessage.Value, "Expected variable [PreviousMessage] not received");
                boErrorOccurred = true;
            } else {
                trTransactionResult.PreviousMessage = aFormVariables.PreviousMessage;
            }
            // cross reference of transaction
            if (isDefined("aFormVariables.CrossReference") NEQ true) {
                szOutputMessage.Value = AddStringToStringList(szOutputMessage.Value, "Expected variable [CrossReference] not received");
                boErrorOccurred = true;
            } else {
                trTransactionResult.CrossReference = aFormVariables.CrossReference;
            }
            // amount (same as value passed into payment form - echoed back out by payment form)
            if (isDefined("aFormVariables.Amount") NEQ true) {
                szOutputMessage.Value = AddStringToStringList(szOutputMessage.Value, "Expected variable [Amount] not received");
                boErrorOccurred = true;
            } else {
            	if (aFormVariables.Amount EQ "")
				{
					trTransactionResult.Amount = "";
				}
				else
				{
					trTransactionResult.Amount = aFormVariables.Amount;
				}
            }
            // currency code (same as value passed into payment form - echoed back out by payment form)
            if (isDefined("aFormVariables.CurrencyCode") NEQ true) {
                szOutputMessage.Value = AddStringToStringList(szOutputMessage.Value, "Expected variable [CurrencyCode] not received");
                boErrorOccurred = true;
            } else {
              	if (aFormVariables.CurrencyCode EQ "")
				{
					trTransactionResult.CurrencyCode = "";
				}
				else
				{
					trTransactionResult.CurrencyCode = aFormVariables.CurrencyCode;
				}
            }
            // order ID (same as value passed into payment form - echoed back out by payment form)
            if (isDefined("aFormVariables.OrderID") NEQ true) {
                szOutputMessage.Value = AddStringToStringList(szOutputMessage.Value, "Expected variable [OrderID] not received");
                boErrorOccurred = true;
            } else {
                trTransactionResult.OrderID = aFormVariables.OrderID;
            }
            // transaction type (same as value passed into payment form - echoed back out by payment form)
            if (isDefined("aFormVariables.TransactionType") NEQ true) {
                szOutputMessage.Value = AddStringToStringList(szOutputMessage.Value, "Expected variable [TransactionType] not received");
                boErrorOccurred = true;
            } else {
                trTransactionResult.TransactionType = aFormVariables.TransactionType;
            }
            // transaction date/time (same as value passed into payment form - echoed back out by payment form)
            if (isDefined("aFormVariables.TransactionDateTime") NEQ true) {
                szOutputMessage.Value = AddStringToStringList(szOutputMessage.Value, "Expected variable [TransactionDateTime] not received");
                boErrorOccurred = true;
            } else {
                trTransactionResult.TransactionDateTime = aFormVariables.TransactionDateTime;
            }
            // order description (same as value passed into payment form - echoed back out by payment form)
            if (isDefined("aFormVariables.OrderDescription") NEQ true) {
                szOutputMessage.Value = AddStringToStringList(szOutputMessage.Value, "Expected variable [OrderDescription] not received");
                boErrorOccurred = true;
            } else {
                trTransactionResult.OrderDescription = aFormVariables.OrderDescription;
            }
            // address1 (not necessarily the same as value passed into payment form - as the customer can change it on the form)
            if (isDefined("aFormVariables.Address1") NEQ true) {
                szOutputMessage.Value = AddStringToStringList(szOutputMessage.Value, "Expected variable [Address1] not received");
                boErrorOccurred = true;
            } else {
                trTransactionResult.Address1 = aFormVariables.Address1;
            }
            // address2 (not necessarily the same as value passed into payment form - as the customer can change it on the form)
            if (isDefined("aFormVariables.Address2") NEQ true) {
                szOutputMessage.Value = AddStringToStringList(szOutputMessage.Value, "Expected variable [Address2] not received");
                boErrorOccurred = true;
            } else {
                trTransactionResult.Address2 = aFormVariables.Address2;
            }
            // address3 (not necessarily the same as value passed into payment form - as the customer can change it on the form)
            if (isDefined("aFormVariables.Address3") NEQ true) {
                szOutputMessage.Value = AddStringToStringList(szOutputMessage.Value, "Expected variable [Address3] not received");
                boErrorOccurred = true;
            } else {
                trTransactionResult.Address3 = aFormVariables.Address3;
            }
            // address4 (not necessarily the same as value passed into payment form - as the customer can change it on the form)
            if (isDefined("aFormVariables.Address4") NEQ true) {
                szOutputMessage.Value = AddStringToStringList(szOutputMessage.Value, "Expected variable [Address4] not received");
                boErrorOccurred = true;
            } else {
                trTransactionResult.Address4 = aFormVariables.Address4;
            }
            // city (not necessarily the same as value passed into payment form - as the customer can change it on the form)
            if (isDefined("aFormVariables.City") NEQ true) {
                szOutputMessage.Value = AddStringToStringList(szOutputMessage.Value, "Expected variable [City] not received");
                boErrorOccurred = true;
            } else {
                trTransactionResult.City = aFormVariables.City;
            }
            // state (not necessarily the same as value passed into payment form - as the customer can change it on the form)
            if (isDefined("aFormVariables.State") NEQ true) {
                szOutputMessage.Value = AddStringToStringList(szOutputMessage.Value, "Expected variable [State] not received");
                boErrorOccurred = true;
            } else {
                trTransactionResult.State = aFormVariables.State;
            }
            // post code (not necessarily the same as value passed into payment form - as the customer can change it on the form)
            if (isDefined("aFormVariables.PostCode") NEQ true) {
                szOutputMessage.Value = AddStringToStringList(szOutputMessage.Value, "Expected variable [PostCode] not received");
                boErrorOccurred = true;
            } else {
                trTransactionResult.PostCode = aFormVariables.PostCode;
            }
            // country code (not necessarily the same as value passed into payment form - as the customer can change it on the form)
            if (isDefined("aFormVariables.CountryCode") NEQ true) {
                szOutputMessage.Value = AddStringToStringList(szOutputMessage.Value, "Expected variable [CountryCode] not received");
                boErrorOccurred = true;
            } else {
            	if (aFormVariables.CountryCode EQ "")
				{
					trTransactionResult.CountryCode = "";
				}
				else
				{
					trTransactionResult.CountryCode = aFormVariables.CountryCode;
				}
            }


            // Address check result - response from bank
            if (isDefined("aFormVariables.AddressNumericCheckResult") NEQ true) {
                szOutputMessage.Value = AddStringToStringList(szOutputMessage.Value, "Expected variable [AddressNumericCheckResult] not received");
                boErrorOccurred = true;
            } else {
                trTransactionResult.AddressNumericCheckResult = aFormVariables.AddressNumericCheckResult;
            }
            // PostCode check result - response from bank
            if (isDefined("aFormVariables.PostCodeCheckResult") NEQ true) {
                szOutputMessage.Value = AddStringToStringList(szOutputMessage.Value, "Expected variable [PostCodeCheckResult] not received");
                boErrorOccurred = true;
            } else {
				trTransactionResult.PostCodeCheckResult = aFormVariables.PostCodeCheckResult;
            }
            // CV2 check result - response from bank
            if (isDefined("aFormVariables.CV2CheckResult") NEQ true) {
                szOutputMessage.Value = AddStringToStringList(szOutputMessage.Value, "Expected variable [CV2CheckResult] not received");
                boErrorOccurred = true;
            } else {
                trTransactionResult.CV2CheckResult = aFormVariables.CV2CheckResult;
            }
            // 3D Secure Authentication check result - response from bank
            if (isDefined("aFormVariables.ThreeDSecureAuthenticationCheckResult") NEQ true) {
                szOutputMessage.Value = AddStringToStringList(szOutputMessage.Value, "Expected variable [ThreeDSecureAuthenticationCheckResult] not received");
                boErrorOccurred = true;
            } else {
                trTransactionResult.ThreeDSecureAuthenticationCheckResult = aFormVariables.ThreeDSecureAuthenticationCheckResult;
            }
            // Card Type - response from bank
            if (isDefined("aFormVariables.CardType") NEQ true) {
                szOutputMessage.Value = AddStringToStringList(szOutputMessage.Value, "Expected variable [CardType] not received");
                boErrorOccurred = true;
            } else {
                trTransactionResult.CardType = aFormVariables.CardType;
            }
            // Card Class - response from bank
            if (isDefined("aFormVariables.CardClass") NEQ true) {
                szOutputMessage.Value = AddStringToStringList(szOutputMessage.Value, "Expected variable [CardClass] not received");
                boErrorOccurred = true;
            } else {
				trTransactionResult.CardClass = aFormVariables.CardClass;
            }
            // Card Issuer - response from bank
            if (isDefined("aFormVariables.CardIssuer") NEQ true) {
                szOutputMessage.Value = AddStringToStringList(szOutputMessage.Value, "Expected variable [CardIssuer] not received");
                boErrorOccurred = true;
            } else {
                trTransactionResult.CardIssuer = aFormVariables.CardIssuer;
            }
            // Card Issuer Country Code - response from bank
            if (isDefined("aFormVariables.CardIssuerCountryCode") NEQ true) {
                szOutputMessage.Value = AddStringToStringList(szOutputMessage.Value, "Expected variable [CardIssuerCountryCode] not received");
                boErrorOccurred = true;
            } else {
                if (aFormVariables.CardIssuerCountryCode EQ "")
				{
					trTransactionResult.CardIssuerCountryCode = "";
				}
				else
				{
					trTransactionResult.CardIssuerCountryCode = aFormVariables.CardIssuerCountryCode;
				}
            }
            // Email Address (not necessarily the same as value passed into payment form - as the customer can change it on the form)
            if (isDefined("aFormVariables.EmailAddress") NEQ true) {
                szOutputMessage.Value = AddStringToStringList(szOutputMessage.Value, "Expected variable [EmailAddress] not received");
                boErrorOccurred = true;
            } else {
                trTransactionResult.EmailAddress = aFormVariables.EmailAddress;
            }
            // Phone Number (not necessarily the same as value passed into payment form - as the customer can change it on the form)
            if (isDefined("aFormVariables.PhoneNumber") NEQ true) {
                szOutputMessage.Value = AddStringToStringList(szOutputMessage.Value, "Expected variable [PhoneNumber] not received");
                boErrorOccurred = true;
            } else {
				trTransactionResult.PhoneNumber = aFormVariables.PhoneNumber;
            }

            if (boErrorOccurred) {
                trTransactionResult = null;
            }
        } catch (Any e) {
            boErrorOccurred = true;
            szOutputMessage.Value = e.getMessage();
        }

        if (boErrorOccurred EQ true)
		{
			boReturnValue = false;
		}
		else
		{
			boReturnValue = true;
		}

		return (boReturnValue);
    }
    
	// You should put your code that does any post transaction tasks
	// (e.g. updates the order object, sends the customer an email etc) in this function
    // This is a "hook" function that is run when the results of the transaction are
    // known.
    // You should put your code that does any post transaction tasks
    // (e.g. updates the order object, sends the customer an email etc) in this function
    function reportTransactionResult(trTransactionResult, szOutputMessage)
    {
        
        boErrorOccurred = false;
		boReturnValue = false;
		
		szOutputMessage.Value = "";
        //szOutputMessage = "Environment specific function reportTransactionResults() needs to be implemented by merchant developer";    
        
        try
        {
            switch (trTransactionResult.StatusCode)
            {
                // transaction authorised
                case 0:
                
                    break;
                // transaction declined
                case 5:
                    
					break;
                // duplicate transaction
                case 20:
                    // need to look at the previous status code to see if the
                    // transaction was successful
                    if (trTransactionResult.PreviousStatusCode EQ 0)
                    {
                        // transaction authorised
                        
                    }
                    else
                    {
                        // transaction not authorised
                        
                    }
                    szPreviousTransactionMessage.Value = trTransactionResult.PreviousMessage;
    
                    break;
                // error occurred
                case 30:
                    
                    break;
                default:
                    
                    break;
            }

            // put code to update/store the order with the transaction result
            
        }  
        catch (Any e)
        {
            boErrorOccurred = true;
            szOutputMessage = e.message;
        }
		if (boErrorOccurred EQ true)
		{
			boReturnValue = false;
		}
		else
		{
			boReturnValue = true;
		}

		return (boReturnValue);
    }
</cfscript>