package net.mycompanyname;

import java.net.NetPermission;
import java.text.SimpleDateFormat;
import java.util.Date;
import javax.servlet.http.HttpServletRequest;

public class PaymentForm {

    public class FORM_MODE {

        public static final int PAYMENT_FORM = 0;
        public static final int THREE_D_SECURE_AUTHENTICATION_REQUIRED = 1;
        public static final int RESULTS = 2;
    }

    public class INCOMING_VARIABLE_SOURCE {

        public static final int SHOPPING_CART_CHECKOUT = 0;
        public static final int THREE_D_SECURE_AUTHENTICATION_REQUIRED = 1;
        public static final int RESULTS = 2;
    }
    private int m_fmFormMode = FORM_MODE.PAYMENT_FORM;
    private boolean m_boIncomingVariableErrors = false;
    protected String m_szHashDigest;
    protected String m_szAmount;
    protected int m_nCurrencyCode;
    protected String m_szOrderID;
    protected String m_szTransactionType;
    protected String m_szTransactionDateTime;
    protected String m_szCallbackURL;
    protected String m_szOrderDescription;
    protected String m_szAddress1;
    protected String m_szAddress2;
    protected String m_szAddress3;
    protected String m_szAddress4;
    protected String m_szCity;
    protected String m_szState;
    protected String m_szPostCode;
    protected int m_nCountryCode;
    protected String m_szCardName;
    protected String m_szCardNumber;
    protected String m_szExpiryDateMonth;
    protected String m_szExpiryDateYear;
    protected String m_szStartDateMonth;
    protected String m_szStartDateYear;
    protected String m_szIssueNumber;
    protected String m_szCV2;
    protected String m_szCrossReference;
    protected String m_szPaRES;
    protected String m_szPaREQ;
    protected String m_szTermURL;
    protected ListItemCollection m_licExpiryDateMonths;
    protected ListItemCollection m_licExpiryDateYears;
    protected ListItemCollection m_licStartDateMonths;
    protected ListItemCollection m_licStartDateYears;
    protected ListItemCollection m_licCountryCodes;
    private TransactionResult trTransactionResult;
    private OutTransactionResult otrOutTransactionResult;
    private OutString outputMessage;
    private OutString incomingHashDigest;
    private String m_szMessageClass = "";
    private String m_szMessage = "";
    private boolean m_boTransactionSuccessful;
    private boolean m_boDuplicateTransaction;
    private SimpleDateFormat dateFormat;
    private NameValueList nameValueList;
    private String m_szPreviousTransactionMessage = "";
    private boolean m_boSuppressFormDisplay = false;
    private String m_szFormAction = "";
    private String m_szFormAttributes = "";
    private String m_szBodyAttributes = "";
    private String m_szFormMode;

    private int m_nFormMode = -1;

    private boolean CheckIntegrityOfIncomingVariables(HttpServletRequest request, Global global, int ivsIncomingVariableSource) {
        boolean boReturnValue = false;
        String szMessage;
        String szStringToHash = null;

        nameValueList = PaymentFormHelper.parseIntoList(request);

        otrOutTransactionResult = new OutTransactionResult();
        incomingHashDigest = new OutString();
        outputMessage = new OutString();

        switch (ivsIncomingVariableSource) {
            case INCOMING_VARIABLE_SOURCE.SHOPPING_CART_CHECKOUT:
                szStringToHash =
                        PaymentFormHelper.generateStringToHash( request.getParameter("Amount"),
                                                                request.getParameter("CurrencyCode"),
                                                                request.getParameter("OrderID"),
                                                                request.getParameter("TransactionType"),
                                                                request.getParameter("TransactionDateTime"),
                                                                request.getParameter("CallbackURL"),
                                                                request.getParameter("OrderDescription"),
                                                                request.getParameter("CustomerName"),
                                                                request.getParameter("Address1"),
                                                                request.getParameter("Address2"),
                                                                request.getParameter("Address3"),
                                                                request.getParameter("Address4"),
                                                                request.getParameter("City"),
                                                                request.getParameter("State"),
                                                                request.getParameter("PostCode"),
                                                                request.getParameter("CountryCode"),
                                                                global.getSecretKey());

                m_szHashDigest = PaymentFormHelper.calculateHashDigest(
                        szStringToHash, global.getSecretKey(), global.getHashMethod());


                if (request.getParameter("ShoppingCartHashDigest").equals(m_szHashDigest)) {
                    boReturnValue = true;
                }

                break;
            case INCOMING_VARIABLE_SOURCE.THREE_D_SECURE_AUTHENTICATION_REQUIRED:

                if (PaymentFormHelper.Get3DSecureAuthenticationRequiredFromPostVariables(
                        nameValueList, otrOutTransactionResult, incomingHashDigest, outputMessage)) {

                    szStringToHash =
                            PaymentFormHelper.GenerateStringToHash3DSecureAuthenticationRequired(
                            global.getMerchantID(), global.getPassword(), otrOutTransactionResult.output, global.getPreSharedKey(), global.getHashMethod());

                    m_szHashDigest = PaymentFormHelper.calculateHashDigest(
                            szStringToHash, global.getPreSharedKey(), global.getHashMethod());

                    if (incomingHashDigest.output.equals(m_szHashDigest)) {
                        trTransactionResult = otrOutTransactionResult.output;
                        boReturnValue = true;
                    }
                }
                break;
            case INCOMING_VARIABLE_SOURCE.RESULTS:

                if (PaymentFormHelper.GetTransactionCompleteResultFromPostVariables(
                        nameValueList, otrOutTransactionResult, incomingHashDigest, outputMessage)) {

                    szStringToHash = PaymentFormHelper.GenerateStringToHashPaymentComplete(
                            global.getMerchantID(), global.getPassword(), otrOutTransactionResult.output, global.getPreSharedKey(), global.getHashMethod());

                    m_szHashDigest = PaymentFormHelper.calculateHashDigest(
                            szStringToHash, global.getPreSharedKey(), global.getHashMethod());

                    if (incomingHashDigest.output.equals(m_szHashDigest)) {
                        trTransactionResult = otrOutTransactionResult.output;
                        boReturnValue = true;
                    }
                }
                break;
        }

        return (boReturnValue);
    }

    public void preProcessPaymentForm(HttpServletRequest request, Global global) {
        boolean boResetFormVariables = false;

        String szStringToHash = "";

        if (request.getContentLength() <= 0) {
            m_fmFormMode = FORM_MODE.RESULTS;
            m_boIncomingVariableErrors = true;
            m_boSuppressFormDisplay = true;

            m_szMessageClass = "ErrorMessage";
            m_szMessage = "No input variables passed";

        } else {
            trTransactionResult = null;
            m_szFormAction = "https://mms." + global.getPaymentProcessorDomain() + "/Pages/PublicPages/TransparentRedirect.aspx";

            if (request.getParameter("FormMode") == null && request.getParameter("ShoppingCartHashDigest") != null) {
                if (!CheckIntegrityOfIncomingVariables(request, global, INCOMING_VARIABLE_SOURCE.SHOPPING_CART_CHECKOUT)) {
                    m_boIncomingVariableErrors = true;
                    m_boSuppressFormDisplay = true;

                    m_fmFormMode = FORM_MODE.RESULTS;

                    m_szMessageClass = "ErrorMessage";
                    m_szMessage = "Variable tampering detected";

                } else {
                    m_szAmount = request.getParameter("Amount");
                    m_nCurrencyCode = Integer.parseInt(request.getParameter("CurrencyCode"));
                    m_szOrderID = request.getParameter("OrderID");
                    m_szOrderDescription = request.getParameter("OrderDescription");

                    // the transaction type - can be SALE or PREAUTH
                    m_szTransactionType = request.getParameter("TransactionType");
                    // the GMT/UTC relative date/time for the transaction (MUST either be in GMT/UTC
                    // or MUST include the correct timezone offset)
                    m_szTransactionDateTime = request.getParameter("TransactionDateTime");

                    m_szCallbackURL = request.getParameter("CallbackURL");
                    // these variables allow the payment form to be "seeded" with initial values
                    m_szCardName = request.getParameter("CustomerName");
                    m_szCardNumber = "";
                    m_szIssueNumber = "";
                    m_szCV2 = "";
                    m_szAddress1 = request.getParameter("Address1");
                    m_szAddress2 = request.getParameter("Address2");
                    m_szAddress3 = request.getParameter("Address3");
                    m_szAddress4 = request.getParameter("Address4");
                    m_szCity = request.getParameter("City");
                    m_szState = request.getParameter("State");
                    m_szPostCode = request.getParameter("PostCode");
                    m_nCountryCode = Integer.parseInt(request.getParameter("CountryCode"));

                    szStringToHash = PaymentFormHelper.GenerateStringToHashInitial(global.getMerchantID(), global.getPassword(), m_szAmount, m_nCurrencyCode, m_szOrderID, m_szTransactionType, m_szTransactionDateTime, m_szCallbackURL, m_szOrderDescription, global.getPreSharedKey(), global.getHashMethod());

                    m_szHashDigest = PaymentFormHelper.calculateHashDigest(szStringToHash, global.getPreSharedKey(), global.getHashMethod());

                }
            } else {
                if (request.getParameter("HashDigest") != null) {
                    if (request.getParameter("PaREQ") != null) {

                        if (!CheckIntegrityOfIncomingVariables(request, global, INCOMING_VARIABLE_SOURCE.THREE_D_SECURE_AUTHENTICATION_REQUIRED)) {
                            m_boIncomingVariableErrors = true;
                            m_boSuppressFormDisplay = true;

                            m_fmFormMode = FORM_MODE.RESULTS;

                            m_szMessageClass = "ErrorMessage";
                            m_szMessage = "Variable tampering detected";

                        } else {
                            m_fmFormMode = FORM_MODE.THREE_D_SECURE_AUTHENTICATION_REQUIRED;

                            m_szFormAction = trTransactionResult.getACSURL();
                            m_szBodyAttributes = " onload=\"document.Form.submit()\"";
                            m_szFormAttributes = " target=\"ACSFrame\"";

                            m_szPaREQ = trTransactionResult.getPaREQ();
                            m_szCrossReference = trTransactionResult.getCrossReference();
                            m_szTermURL = PaymentFormHelper.getSiteSecureBaseURL(request) + "ThreeDSecureLandingPage.aspx";

                        }
                    } else {
                        m_fmFormMode = FORM_MODE.RESULTS;

                        if (!CheckIntegrityOfIncomingVariables(request, global, INCOMING_VARIABLE_SOURCE.RESULTS)) {
                            m_boIncomingVariableErrors = true;
                            m_boSuppressFormDisplay = true;

                            m_szMessageClass = "ErrorMessage";
                            m_szMessage = "Variable tampering detected";

                        } else {

                            outputMessage = new OutString();

                            if (!PaymentFormHelper.reportTransactionResults(otrOutTransactionResult.output, outputMessage))
                            {
                                // handle the case where the results aren't stored correctly
                                m_szMessageClass = "ErrorMessage";

                                // You might want to display the actual exception message during testing
                                // If so, comment and uncomment out the two lines below as necessary
                                m_szMessage = "An error has occured";
                                //m_szMessage = szOutputMessage;
                            }
                            else
                            {
                                m_boSuppressFormDisplay = true;

                                switch (otrOutTransactionResult.output.getStatusCode()) {
                                    // transaction authorised
                                    case 0:
                                        m_szMessageClass = "SuccessMessage";
                                        break;
                                    // transaction declined
                                    case 5:
                                        m_szMessageClass = "ErrorMessage";
                                        break;
                                    // duplicate transaction
                                    case 20:
                                        m_boDuplicateTransaction = true;

                                        // need to look at the previous status code to see if the
                                        // transaction was successful
                                        try {

                                            if (otrOutTransactionResult.output.getPreviousStatusCode().getValue() == 0) {
                                                m_szMessageClass = "SuccessMessage";
                                                // transaction authorised
                                            } else {
                                                m_szMessageClass = "ErrorMessage";
                                                // transaction not authorised
                                            }
                                        } catch (Exception e) {
                                        }
                                        m_szPreviousTransactionMessage = otrOutTransactionResult.output.getPreviousMessage();

                                        break;
                                    // error occurred
                                    case 30:
                                        m_szMessageClass = "ErrorMessage";
                                        break;
                                    default:
                                        m_szMessageClass = "ErrorMessage";
                                        break;
                                }
                                m_szMessage = otrOutTransactionResult.output.getMessage();
                            }
                        }
                    }
                }
            }
        }

        // Reset the form variables if required
        if (boResetFormVariables == true) {
            m_szCardName = "";
            m_szCardNumber = "";
            m_szExpiryDateMonth = "";
            m_szExpiryDateYear = "";
            m_szStartDateMonth = "";
            m_szStartDateYear = "";
            m_szIssueNumber = "";
            m_szCV2 = "";
            m_szAddress1 = "";
            m_szAddress2 = "";
            m_szAddress3 = "";
            m_szAddress4 = "";
            m_szCity = "";
            m_szState = "";
            m_szPostCode = "";
        }
    }

    public String getFormMode(int formMode) {
        String returnMode = "";

        switch (formMode) {
            case FORM_MODE.PAYMENT_FORM:
                returnMode = "PAYMENT_FORM";
                break;
            case FORM_MODE.RESULTS:
                returnMode = "RESULTS";
                break;
            case FORM_MODE.THREE_D_SECURE_AUTHENTICATION_REQUIRED:
                returnMode = "THREE_D_SECURE_AUTHENTICATION_REQUIRED";
                break;
        }

        return (returnMode);
    }

    public int getFormMode(String formMode) {
        int returnMode = -1;

        if (formMode != null
                && !formMode.equals("")) {
            if (formMode.equalsIgnoreCase("PAYMENT_FORM")) {
                returnMode = FORM_MODE.PAYMENT_FORM;
            }
            if (formMode.equalsIgnoreCase("RESULTS")) {
                returnMode = FORM_MODE.RESULTS;
            }
            if (formMode.equalsIgnoreCase("THREE_D_SECURE_AUTHENTICATION_REQUIRED")) {
                returnMode = FORM_MODE.THREE_D_SECURE_AUTHENTICATION_REQUIRED;
            }
        }
        return (returnMode);
    }

    public String getHashDigest() {
        return m_szHashDigest;
    }

    public String getAmount() {
        return m_szAmount;
    }

    public int getCurrencyCode() {
        return m_nCurrencyCode;
    }

    public String getOrderID() {
        return m_szOrderID;
    }

    public String getTransactionType() {
        return m_szTransactionType;
    }

    public String getTransactionDateTime() {
        return m_szTransactionDateTime;
    }

    public String getCallbackURL() {
        return m_szCallbackURL;
    }

    public String getOrderDescription() {
        return m_szOrderDescription;
    }

    public String getAddress1() {
        return m_szAddress1;
    }

    public String getAddress2() {
        return m_szAddress2;
    }

    public String getAddress3() {
        return m_szAddress3;
    }

    public String getAddress4() {
        return m_szAddress4;
    }

    public String getCity() {
        return m_szCity;
    }

    public String getState() {
        return m_szState;
    }

    public String getPostCode() {
        return m_szPostCode;
    }

    public int getCountryCode() {
        return m_nCountryCode;
    }

    public String getCardName() {
        return m_szCardName;
    }

    public String getCardNumber() {
        return m_szCardNumber;
    }

    public String getExpiryDateMonth() {
        return m_szExpiryDateMonth;
    }

    public String getExpiryDateYear() {
        return m_szExpiryDateYear;
    }

    public String getStartDateMonth() {
        return m_szStartDateMonth;
    }

    public String getStartDateYear() {
        return m_szStartDateYear;
    }

    public String getIssueNumber() {
        return m_szIssueNumber;
    }

    public String getCV2() {
        return m_szCV2;
    }

    public String getCrossReference() {
        return m_szCrossReference;
    }

    public String getPaRES() {
        return m_szPaRES;
    }

    public String getPaREQ() {
        return m_szPaREQ;
    }

    public String getTermURL() {
        return m_szTermURL;
    }

    public String getFormModeString() {
        return getFormMode(m_fmFormMode);
    }

    public int getFormModeEnum() {
        return m_fmFormMode;
    }

    public String getFormAction() {
        return m_szFormAction;
    }

    public String getFormAttributes() {
        return m_szFormAttributes;
    }

    public String getBodyAttributes() {
        return m_szBodyAttributes;
    }

    public String getMessage() {
        return m_szMessage;
    }

    public String getMessageClass() {
        return m_szMessageClass;
    }

    public boolean getDuplicateTransaction() {
        return m_boDuplicateTransaction;
    }

    public boolean getTransactionSuccessful() {
        return m_boTransactionSuccessful;
    }

    public String getPreviousTransactionMessage() {
        return m_szPreviousTransactionMessage;
    }

    public boolean getSuppressFormDisplay() {
        return m_boSuppressFormDisplay;
    }
}
