using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;

namespace MyCompanyName
{
    public partial class PaymentForm : System.Web.UI.Page
    {
        protected enum FORM_MODE
        {
            PAYMENT_FORM,
            THREE_D_SECURE_AUTHENTICATION_REQUIRED,
            RESULTS
        }
        protected enum INCOMING_VARIABLE_SOURCE
        {
            SHOPPING_CART_CHECKOUT,
            THREE_D_SECURE_AUTHENTICATION_REQUIRED,
            RESULTS
        }


        protected FORM_MODE m_fmFormMode = FORM_MODE.PAYMENT_FORM;
        private bool m_boIncomingVariableErrors = false;

        protected internal string m_szPaRES;
        protected internal string m_szPaREQ;
        protected internal string m_szTermURL;
        protected internal string m_szACSURL;
        protected internal string m_szCrossReference;

        protected string m_szFormAction;
        protected string m_szHashDigest;
        protected string m_szMerchantID;
        protected string m_szAmount;
        protected string m_szCurrencyCode;
        protected string m_szOrderID;
        protected string m_szTransactionType;
        protected string m_szTransactionDateTime;
        protected string m_szCallbackURL;
        protected string m_szOrderDescription;
        protected string m_szCustomerName;
        protected string m_szAddress1;
        protected string m_szAddress2;
        protected string m_szAddress3;
        protected string m_szAddress4;
        protected string m_szCity;
        protected string m_szState;
        protected string m_szPostCode;
        protected string m_szCountryCode;

        protected string m_szCardNumber;
        protected string m_szExpiryDateMonth;
        protected string m_szExpiryDateYear;
        protected string m_szStartDateMonth;
        protected string m_szStartDateYear;
        protected string m_szIssueNumber;
        protected string m_szCV2;


        protected ListItemCollection m_licExpiryDateMonths;
        protected ListItemCollection m_licExpiryDateYears;

        protected ListItemCollection m_licStartDateMonths;
        protected ListItemCollection m_licStartDateYears;

        protected ListItemCollection m_licCountryCodes;

        private TransactionResult trTransactionResult;

        private string szOutputMessage;
        protected string m_szMessage;
        protected string m_szMessageClass;
        protected bool m_boDuplicateTransaction;
        protected string m_szPreviousTransactionMessage;


        protected void Page_Load(object sender, EventArgs e)
        {
            trTransactionResult = null;

            m_szFormAction = "https://mms." + Global.PaymentProcessorDomain + "/Pages/PublicPages/TransparentRedirect.aspx";

            if (Request.Form.Count == 0)
            {
                m_fmFormMode = FORM_MODE.RESULTS;
                m_boIncomingVariableErrors = true;

                pnMessagePanel.CssClass = "ErrorMessage";
                lbMessage.Text = "No input variables passed";
                pnMessagePanel.Visible = true;

            }
            else
            {
                if (Request.Form["ShoppingCartHashDigest"] != null)
                {
                    if (!CheckIntegrityOfIncomingVariables(INCOMING_VARIABLE_SOURCE.SHOPPING_CART_CHECKOUT))
                    {
                        m_boIncomingVariableErrors = true;
                        m_fmFormMode = FORM_MODE.RESULTS;

                        pnMessagePanel.CssClass = "ErrorMessage";
                        lbMessage.Text = "Variable tampering detected";
                        pnMessagePanel.Visible = true;
                    }
                    else
                    {
                        m_szFormAction = "https://mms." + Global.PaymentProcessorDomain + "/Pages/PublicPages/TransparentRedirect.aspx";

                        m_szMerchantID = Global.MerchantID;
                        // the transaction type - can be SALE or PREAUTH
                        m_szTransactionType = Request.Form["TransactionType"];
                        // the GMT/UTC relative date/time for the transaction (MUST either be in GMT/UTC 
                        // or MUST include the correct timezone offset)
                        m_szTransactionDateTime = Request.Form["TransactionDateTime"];

                        m_szCallbackURL = Request.Form["CallbackURL"];
                        // these variables allow the payment form to be "seeded" with initial values
                        m_szCustomerName = Request.Form["CustomerName"];
                        m_szAddress1 = Request.Form["Address1"];
                        m_szAddress2 = Request.Form["Address2"];
                        m_szAddress3 = Request.Form["Address3"];
                        m_szAddress4 = Request.Form["Address4"];
                        m_szCity = Request.Form["City"];
                        m_szState = Request.Form["State"];
                        m_szPostCode = Request.Form["PostCode"];
                        m_szCurrencyCode = Request.Form["CurrencyCode"];
                    }

                }
                else
                {
                    if (Request.Form["HashDigest"] != null)
                    {
                        if (Request.Form["PaREQ"] != null)
                        {
                            if (!CheckIntegrityOfIncomingVariables(INCOMING_VARIABLE_SOURCE.THREE_D_SECURE_AUTHENTICATION_REQUIRED))
                            {
                                m_boIncomingVariableErrors = true;
                                m_fmFormMode = FORM_MODE.RESULTS;

                                pnMessagePanel.CssClass = "ErrorMessage";
                                lbMessage.Text = "Variable tampering detected";
                                pnMessagePanel.Visible = true;
                            }
                            else
                            {
                                m_fmFormMode = FORM_MODE.THREE_D_SECURE_AUTHENTICATION_REQUIRED;

                                m_szPaREQ = trTransactionResult.PaREQ;
                                m_szACSURL = trTransactionResult.ACSUrl;
                                m_szCrossReference = trTransactionResult.CrossReference;
                                m_szTermURL = PaymentFormHelper.GetSiteSecureBaseURL(Request) + "ThreeDSecureLandingPage.aspx";

                            }
                        }
                        else
                        {
                            m_fmFormMode = FORM_MODE.RESULTS;

                            pnMessagePanel.Visible = true;

                            if (!CheckIntegrityOfIncomingVariables(INCOMING_VARIABLE_SOURCE.RESULTS))
                            {
                                m_boIncomingVariableErrors = true;

                                pnMessagePanel.CssClass = "ErrorMessage";
                                lbMessage.Text = "Variable tampering detected";
                            }
                            else
                            {
                                if (!PaymentFormHelper.ReportTransactionResult(trTransactionResult, out szOutputMessage))
                                {
                                    // handle the case where the results aren't stored correctly
                                    pnMessagePanel.CssClass = "ErrorMessage";

                                    // You might want to display the actual exception message during testing
                                    // If so, comment and uncomment out the two lines below as necessary
                                    lbMessage.Text = "An error has occured";
                                    //lbMessage.Text = szOutputMessage;
                                }
                                else
                                {
                                    switch (trTransactionResult.StatusCode)
                                    {
                                        // transaction authorised
                                        case 0:
                                            pnMessagePanel.CssClass = "SuccessMessage";
                                            break;
                                        // transaction declined
                                        case 5:
                                            pnMessagePanel.CssClass = "ErrorMessage";
                                            break;
                                        // duplicate transaction
                                        case 20:
                                            pnDuplicateTransactionPanel.Visible = true;

                                            // need to look at the previous status code to see if the
                                            // transaction was successful
                                            if (trTransactionResult.PreviousStatusCode == 0)
                                            {
                                                // transaction authorised
                                                pnMessagePanel.CssClass = "SuccessMessage";
                                            }
                                            else
                                            {
                                                // transaction not authorised
                                                pnMessagePanel.CssClass = "ErrorMessage";
                                            }
                                            lbPreviousTransactionMessage.Text = trTransactionResult.PreviousMessage;

                                            break;
                                        // error occurred
                                        case 30:
                                            pnMessagePanel.CssClass = "ErrorMessage";
                                            break;
                                        default:
                                            pnMessagePanel.CssClass = "ErrorMessage";
                                            break;
                                    }
                                    lbMessage.Text = trTransactionResult.Message;
                                }
                            }
                        }
                    }
                }
                PopulateFormItems();
            }
        }

        private bool CheckIntegrityOfIncomingVariables(INCOMING_VARIABLE_SOURCE ivsIncomingVariableSource)
        {
            bool boReturnValue = false;
            string szMessage;
            string szStringToHash;

            switch (ivsIncomingVariableSource)
            {
                case INCOMING_VARIABLE_SOURCE.SHOPPING_CART_CHECKOUT:
                    szStringToHash = 
                        PaymentFormHelper.GenerateStringToHash(
                            Request.Form["Amount"],
                            Request.Form["CurrencyCode"],
                            Request.Form["OrderID"],
                            Request.Form["TransactionType"],
                            Request.Form["TransactionDateTime"],
                            Request.Form["CallbackURL"],
                            Request.Form["OrderDescription"],
                            Request.Form["CustomerName"],
                            Request.Form["Address1"],
                            Request.Form["Address2"],
                            Request.Form["Address3"],
                            Request.Form["Address4"],
                            Request.Form["City"],
                            Request.Form["State"],
                            Request.Form["PostCode"],
                            Request.Form["CountryCode"],
                            Global.SecretKey);

                    if (Request.Form["ShoppingCartHashDigest"] ==
                        PaymentFormHelper.CalculateHashDigest(szStringToHash, MyCompanyName.Global.SecretKey, MyCompanyName.Global.HashMethod))
                    {
                        boReturnValue = true;
                    }

                    break;
                case INCOMING_VARIABLE_SOURCE.THREE_D_SECURE_AUTHENTICATION_REQUIRED:

                    szStringToHash = null;

                    if (PaymentFormHelper.Get3DSecureAuthenticationRequiredFromPostVariables(Request.Form, out trTransactionResult, out m_szHashDigest, out szMessage))
                    {
                        szStringToHash = PaymentFormHelper.GenerateStringToHash3DSecureAuthenticationRequired(Global.MerchantID, Global.Password, trTransactionResult, Global.PreSharedKey, Global.HashMethod);

                        if (m_szHashDigest ==
                            PaymentFormHelper.CalculateHashDigest(szStringToHash, MyCompanyName.Global.PreSharedKey, MyCompanyName.Global.HashMethod))
                        {
                            boReturnValue = true;
                        }
                    }
                    break;
                case INCOMING_VARIABLE_SOURCE.RESULTS:

                    if (PaymentFormHelper.getTransactionCompleteResultFromPostVariables(Request.Form, out trTransactionResult, out m_szHashDigest, out szMessage))
                    {
                        szStringToHash = PaymentFormHelper.GenerateStringToHashPaymentComplete(Global.MerchantID, Global.Password, trTransactionResult, Global.PreSharedKey, Global.HashMethod);
                        if (m_szHashDigest == PaymentFormHelper.CalculateHashDigest(szStringToHash, Global.PreSharedKey, Global.HashMethod))
                        {
                            boReturnValue = true;
                        }
                    }
                    break;
                default:
                    throw new Exception("Invalid incoming variable source: " + ivsIncomingVariableSource.ToString());
            }

            return (boReturnValue);
        }

        protected override void OnPreRender(EventArgs e)
        {
            base.OnPreRender(e);

            // need to work out what mode to put the form into
            switch (m_fmFormMode)
            {
                case FORM_MODE.PAYMENT_FORM:
                    PaymentFormMultiView.SetActiveView(PaymentFormView);
                    break;
                case FORM_MODE.THREE_D_SECURE_AUTHENTICATION_REQUIRED:
                    PaymentFormMultiView.SetActiveView(ThreeDSecureAuthenticationRequired);

                    ACSFrame.Attributes["src"] = PaymentFormHelper.GetSiteSecureBaseURL(Request) + "Loading.htm";
                    BodyTag.Attributes["onload"] += "document.ACSForm.submit();";
                    break;

                case FORM_MODE.RESULTS:
                    PaymentFormMultiView.SetActiveView(ResultsView);
                    break;
                default:
                    throw new Exception("Invalid form mode: " + m_fmFormMode.ToString());
            }
        }

        private void PopulateFormItems()
        {
            string szWorkingValue;
            string szWorkingDisplayValue;
            int nCount = 0;
            int nCurrentYear;
            ListItem liListItem;
            string szStringToHash;

            if (m_boIncomingVariableErrors == true)
            {
                return;
            }

            switch (m_fmFormMode)
            {
                case FORM_MODE.PAYMENT_FORM:

                    m_licExpiryDateMonths = new ListItemCollection();
                    m_licExpiryDateYears = new ListItemCollection();
                    m_licStartDateMonths = new ListItemCollection();
                    m_licStartDateYears = new ListItemCollection();
                    m_licCountryCodes = new ListItemCollection();

                    // read the values from the shopping cart checkout
                    m_szAmount = Request.Form["Amount"];
                    m_szCurrencyCode = Request.Form["CurrencyCode"];
                    m_szOrderID = Request.Form["OrderID"];
                    m_szOrderDescription = Request.Form["OrderDescription"];
                    
                    // build the has digest for this form
                    szStringToHash = PaymentFormHelper.GenerateStringToHashInitial(Global.MerchantID, Global.Password, m_szAmount, m_szCurrencyCode, m_szOrderID, m_szTransactionType, m_szTransactionDateTime, m_szCallbackURL, m_szOrderDescription, Global.PreSharedKey, Global.HashMethod);

                    m_szHashDigest = PaymentFormHelper.CalculateHashDigest(szStringToHash, MyCompanyName.Global.PreSharedKey, MyCompanyName.Global.HashMethod);


                    lbAmount.Text = m_szAmount;
                    lbOrderID.Text = m_szOrderID;
                    lbOrderDescription.Text = m_szOrderDescription;

                    m_licCountryCodes.Add("");

                    // TODO ISOCOUNTRY LOOP
                    liListItem = new ListItem("United Kingdom", "826");
                    liListItem.Selected = true;
                    m_licCountryCodes.Add(liListItem);
                    
                    nCurrentYear = DateTime.Now.Year;
                    
                    m_licExpiryDateYears.Add("");
                    m_licExpiryDateMonths.Add("");

                    m_licStartDateYears.Add("");
                    m_licStartDateMonths.Add("");

                    for (nCount = 0; nCount < 10; nCount++)
                    {
                        // expiry date - start with this year & add 10
                        szWorkingDisplayValue = System.Convert.ToString(nCurrentYear + nCount);
                        szWorkingValue = szWorkingDisplayValue.Substring(2);
                        liListItem = new ListItem(szWorkingDisplayValue, szWorkingValue);
                        m_licExpiryDateYears.Add(liListItem);

                        // start date - start with this year & minus 10
                        szWorkingDisplayValue = System.Convert.ToString(nCurrentYear - nCount);
                        szWorkingValue = szWorkingDisplayValue.Substring(2);
                        liListItem = new ListItem(szWorkingDisplayValue, szWorkingValue);
                        m_licStartDateYears.Add(liListItem);
                    }

                    for (nCount = 1; nCount <= 12; nCount++)
                    {
                        szWorkingValue = System.Convert.ToString(nCount);
                        szWorkingDisplayValue = MyCompanyName.PaymentFormHelper.ForwardPaddedNumberString(nCount, 2, '0');
                        liListItem = new ListItem(szWorkingDisplayValue, szWorkingValue);
                        m_licExpiryDateMonths.Add(liListItem);

                        liListItem = new ListItem(szWorkingDisplayValue, szWorkingValue);
                        m_licStartDateMonths.Add(liListItem);
                    }
                    break;
            }
        }
        protected string ListItemCollectionToString(ListItemCollection licListItemCollection)
        {
            StringBuilder sb = new StringBuilder();

            foreach (ListItem liListItem in licListItemCollection)
            {
                sb.AppendFormat("<option value=\"{0}\"", liListItem.Value);
                if (liListItem.Selected)
                {
                    sb.Append(" selected");
                }
                sb.AppendFormat(" >{0}</option>", liListItem.Text);
            }

            return sb.ToString();
        }
    }
}
